@@ -14,12 +14,12 @@
 """
 This module contains the :class:`Device` abstract base class.
 """
-# pylint: disable=too-many-format-args, use-maxsplit-arg
+# pylint: disable=too-many-format-args, use-maxsplit-arg, protected-access
 import abc
 import types
 import warnings
-from collections.abc import Iterable, Sequence
 from collections import OrderedDict, namedtuple
+from collections.abc import Iterable, Sequence
 from functools import lru_cache
 
 import numpy as np
@@ -28,12 +28,16 @@ import pennylane as qml
 from pennylane.operation import (
     Operation,
     Observable,
+    Tensor,
+)
+from pennylane.measurements import (
     Sample,
     State,
     Variance,
     Expectation,
     Probability,
-    Tensor,
+    MidMeasure,
+    ShadowExpval,
 )
 from pennylane.wires import Wires, WireError
 
@@ -94,6 +98,15 @@ def _process_shot_sequence(shot_list):
     return total_shots, shot_vector
 
 
+def _get_num_copies(shot_vector):
+    """Helper function to determine the number of copies from a shot vector Sequence(int) or Sequence(ShotTuple)."""
+    if any(isinstance(shot_comp, ShotTuple) for shot_comp in shot_vector):
+        len_shot_vec = sum(shot_v.copies for shot_v in shot_vector)
+    else:
+        len_shot_vec = len(shot_vector)
+    return len_shot_vec
+
+
 class DeviceError(Exception):
     """Exception raised by a :class:`~.pennylane._device.Device` when it encounters an illegal
     operation in the quantum circuit.
@@ -111,8 +124,8 @@ class Device(abc.ABC):
             expectation values of observables. Defaults to 1000 if not specified.
     """
 
-    # pylint: disable=too-many-public-methods
-    _capabilities = {"model": None}
+    # pylint: disable=too-many-public-methods,too-many-instance-attributes
+    _capabilities = {"model": None, "supports_broadcasting": False}
     """The capabilities dictionary stores the properties of a device. Devices can add their
     own custom properties and overwrite existing ones by overriding the ``capabilities()`` method."""
 
@@ -145,20 +158,18 @@ class Device(abc.ABC):
 
     def __repr__(self):
         """String representation."""
-        return "<{} device (wires={}, shots={}) at {}>".format(
-            self.__class__.__name__, self.num_wires, self.shots, hex(id(self))
-        )
+        return f"<{self.__class__.__name__} device (wires={self.num_wires}, shots={self.shots}) at {hex(id(self))}>"
 
     def __str__(self):
         """Verbose string representation."""
-        return "{}\nShort name: {}\nPackage: {}\nPlugin version: {}\nAuthor: {}\nWires: {}\nShots: {}".format(
-            self.name,
-            self.short_name,
-            self.__module__.split(".")[0],
-            self.version,
-            self.author,
-            self.num_wires,
-            self.shots,
+        package = self.__module__.split(".")[0]
+        return (
+            f"{self.name}\nShort name: {self.short_name}\n"
+            f"Package: {package}\n"
+            f"Plugin version: {self.version}\n"
+            f"Author: {self.author}\n"
+            f"Wires: {self.num_wires}\n"
+            f"Shots: {self.shots}"
         )
 
     @property
@@ -253,12 +264,13 @@ class Device(abc.ABC):
             # device is in analytic mode
             self._shots = shots
             self._shot_vector = None
+            self._raw_shot_sequence = None
 
         elif isinstance(shots, int):
             # device is in sampling mode (unbatched)
             if shots < 1:
                 raise DeviceError(
-                    "The specified number of shots needs to be at least 1. Got {}.".format(shots)
+                    f"The specified number of shots needs to be at least 1. Got {shots}."
                 )
 
             self._shots = shots
@@ -267,6 +279,7 @@ class Device(abc.ABC):
         elif isinstance(shots, Sequence) and not isinstance(shots, str):
             # device is in batched sampling mode
             self._shots, self._shot_vector = _process_shot_sequence(shots)
+            self._raw_shot_sequence = shots
 
         else:
             raise DeviceError(
@@ -335,6 +348,30 @@ class Device(abc.ABC):
         wire_map = zip(wires, consecutive_wires)
         return OrderedDict(wire_map)
 
+    def order_wires(self, subset_wires):
+        """Given some subset of device wires return a Wires object with the same wires;
+        sorted according to the device wire map.
+
+        Args:
+            subset_wires (Wires): The subset of device wires (in any order).
+
+        Raise:
+            ValueError: Could not find some or all subset wires subset_wires in device wires device_wires.
+
+        Return:
+            ordered_wires (Wires): a new Wires object containing the re-ordered wires set
+        """
+        subset_lst = subset_wires.tolist()
+
+        try:
+            ordered_subset_lst = sorted(subset_lst, key=lambda label: self.wire_map[label])
+        except KeyError as e:
+            raise ValueError(
+                f"Could not find some or all subset wires {subset_wires} in device wires {self.wires}"
+            ) from e
+
+        return Wires(ordered_subset_lst)
+
     @lru_cache()
     def map_wires(self, wires):
         """Map the wire labels of wires using this device's wire map.
@@ -349,9 +386,7 @@ class Device(abc.ABC):
             mapped_wires = wires.map(self.wire_map)
         except WireError as e:
             raise WireError(
-                "Did not find some of the wires {} on device with wires {}.".format(
-                    wires, self.wires
-                )
+                f"Did not find some of the wires {wires} on device with wires {self.wires}."
             ) from e
 
         return mapped_wires
@@ -378,8 +413,8 @@ class Device(abc.ABC):
         """
         return cls._capabilities
 
-    # pylint: disable=too-many-branches
-    def execute(self, queue, observables, parameters={}, **kwargs):
+    # pylint: disable=too-many-branches,unused-argument
+    def execute(self, queue, observables, parameters=None, **kwargs):
         """Execute a queue of quantum operations on the device and then measure the given observables.
 
         For plugin developers: Instead of overwriting this, consider implementing a suitable subset of
@@ -406,7 +441,8 @@ class Device(abc.ABC):
         self._op_queue = queue
         self._obs_queue = observables
         self._parameters = {}
-        self._parameters.update(parameters)
+        if parameters is not None:
+            self._parameters.update(parameters)
 
         results = []
         if self._shot_vector is not None:
@@ -450,7 +486,7 @@ class Device(abc.ABC):
 
                 elif obs.return_type is not None:
                     raise qml.QuantumFunctionError(
-                        "Unsupported return type specified for observable {}".format(obs.name)
+                        f"Unsupported return type specified for observable {obs.name}"
                     )
 
             self.post_measure()
@@ -624,9 +660,15 @@ class Device(abc.ABC):
             .QuantumTape: The expanded/decomposed circuit, such that the device
             will natively support all operations.
         """
-        obs_on_same_wire = len(
-            circuit._obs_sharing_wires  # pylint: disable=protected-access
-        ) > 0 and not self.supports_observable("Hamiltonian")
+        # pylint: disable=protected-access
+
+        comp_basis_sampled_multi_measure = (
+            len(circuit.measurements) > 1 and circuit.samples_computational_basis
+        )
+        obs_on_same_wire = len(circuit._obs_sharing_wires) > 0 or comp_basis_sampled_multi_measure
+        obs_on_same_wire &= not any(
+            isinstance(o, qml.Hamiltonian) for o in circuit._obs_sharing_wires
+        )
 
         ops_not_supported = not all(self.stopping_condition(op) for op in circuit.operations)
 
@@ -684,11 +726,6 @@ class Device(abc.ABC):
             the sequence of circuits to be executed, and a post-processing function
             to be applied to the list of evaluated circuit results.
         """
-
-        # If the observable contains a Hamiltonian and the device does not
-        # support Hamiltonians, or if the simulation uses finite shots, or
-        # if the Hamiltonian explicitly specifies an observable grouping,
-        # split tape into multiple tapes of diagonalizable known observables.
         supports_hamiltonian = self.supports_observable("Hamiltonian")
         finite_shots = self.shots is not None
         grouping_known = all(
@@ -699,17 +736,67 @@ class Device(abc.ABC):
 
         hamiltonian_in_obs = "Hamiltonian" in [obs.name for obs in circuit.observables]
 
-        if hamiltonian_in_obs and ((not supports_hamiltonian or finite_shots) or grouping_known):
+        return_types = [m.return_type for m in circuit.observables]
+
+        is_shadow = ShadowExpval in return_types
+
+        if hamiltonian_in_obs and (
+            (not supports_hamiltonian or (finite_shots and not is_shadow)) or grouping_known
+        ):
+            # If the observable contains a Hamiltonian and the device does not
+            # support Hamiltonians, or if the simulation uses finite shots, or
+            # if the Hamiltonian explicitly specifies an observable grouping,
+            # split tape into multiple tapes of diagonalizable known observables.
             try:
-                return qml.transforms.hamiltonian_expand(circuit, group=False)
+                circuits, hamiltonian_fn = qml.transforms.hamiltonian_expand(circuit, group=False)
 
             except ValueError as e:
                 raise ValueError(
                     "Can only return the expectation of a single Hamiltonian observable"
                 ) from e
+        elif (
+            len(circuit._obs_sharing_wires) > 0
+            and not hamiltonian_in_obs
+            and all(
+                t not in return_types
+                for t in [
+                    qml.measurements.Sample,
+                    qml.measurements.Probability,
+                    qml.measurements.Counts,
+                    qml.measurements.AllCounts,
+                ]
+            )
+        ):
+            # Check for case of non-commuting terms and that there are no Hamiltonians
+            # TODO: allow for Hamiltonians in list of observables as well.
+            circuits, hamiltonian_fn = qml.transforms.split_non_commuting(circuit)
 
-        # otherwise, return an identity transform
-        return [circuit], lambda res: res[0]
+        else:
+            # otherwise, return the output of an identity transform
+            circuits = [circuit]
+
+            def hamiltonian_fn(res):
+                return res[0]
+
+        # Check whether the circuit was broadcasted (then the Hamiltonian-expanded
+        # ones will be as well) and whether broadcasting is supported
+        if circuit.batch_size is None or self.capabilities().get("supports_broadcasting"):
+            # If the circuit wasn't broadcasted or broadcasting is supported, no action required
+            return circuits, hamiltonian_fn
+
+        # Expand each of the broadcasted Hamiltonian-expanded circuits
+        expanded_tapes, expanded_fn = qml.transforms.map_batch_transform(
+            qml.transforms.broadcast_expand, circuits
+        )
+
+        # Chain the postprocessing functions of the broadcasted-tape expansions and the Hamiltonian
+        # expansion. Note that the application order is reversed compared to the expansion order,
+        # i.e. while we first applied `hamiltonian_expand` to the tape, we need to process the
+        # results from the broadcast expansion first.
+        def total_processing(results):
+            return hamiltonian_fn(expanded_fn(results))
+
+        return expanded_tapes, total_processing
 
     @property
     def op_queue(self):
@@ -813,14 +900,14 @@ class Device(abc.ABC):
             ValueError: if `operation` is not a :class:`~.Operation` class or string
 
         Returns:
-            bool: ``True`` iff supplied operation is supported
+            bool: ``True`` if supplied operation is supported
         """
         if isinstance(operation, type) and issubclass(operation, Operation):
             return operation.__name__ in self.operations
         if isinstance(operation, str):
 
-            if operation.endswith(Operation.string_for_inverse):
-                in_ops = operation[: -len(Operation.string_for_inverse)] in self.operations
+            if operation.endswith(".inv"):
+                in_ops = operation[:-4] in self.operations
                 # TODO: update when all capabilities keys changed to "supports_inverse_operations"
                 supports_inv = self.capabilities().get(
                     "supports_inverse_operations", False
@@ -851,8 +938,8 @@ class Device(abc.ABC):
         if isinstance(observable, str):
 
             # This check regards observables that are also operations
-            if observable.endswith(Operation.string_for_inverse):
-                return self.supports_operation(observable[: -len(Operation.string_for_inverse)])
+            if observable.endswith(".inv"):
+                return self.supports_operation(observable[:-4])
 
             return observable in self.observables
 
@@ -879,26 +966,33 @@ class Device(abc.ABC):
 
             operation_name = o.name
 
-            if o.inverse:
+            if getattr(o, "return_type", None) == MidMeasure and not self.capabilities().get(
+                "supports_mid_measure", False
+            ):
+                raise DeviceError(
+                    f"Mid-circuit measurements are not natively supported on device {self.short_name}. "
+                    "Apply the @qml.defer_measurements decorator to your quantum function to "
+                    "simulate the application of mid-circuit measurements on this device."
+                )
+
+            if getattr(o, "inverse", False):
                 # TODO: update when all capabilities keys changed to "supports_inverse_operations"
                 supports_inv = self.capabilities().get(
                     "supports_inverse_operations", False
                 ) or self.capabilities().get("inverse_operations", False)
                 if not supports_inv:
                     raise DeviceError(
-                        "The inverse of gates are not supported on device {}".format(
-                            self.short_name
-                        )
+                        f"The inverse of gates are not supported on device {self.short_name}"
                     )
                 operation_name = o.base_name
 
-            if not self.supports_operation(operation_name):
+            if not self.stopping_condition(o):
                 raise DeviceError(
-                    "Gate {} not supported on device {}".format(operation_name, self.short_name)
+                    f"Gate {operation_name} not supported on device {self.short_name}"
                 )
 
         for o in observables:
-            if isinstance(o, qml.measure.MeasurementProcess) and o.obs is not None:
+            if isinstance(o, qml.measurements.MeasurementProcess) and o.obs is not None:
                 o = o.obs
 
             if isinstance(o, Tensor):
@@ -908,15 +1002,13 @@ class Device(abc.ABC):
                 ) or self.capabilities().get("tensor_observables", False)
                 if not supports_tensor:
                     raise DeviceError(
-                        "Tensor observables not supported on device {}".format(self.short_name)
+                        f"Tensor observables not supported on device {self.short_name}"
                     )
 
                 for i in o.obs:
                     if not self.supports_observable(i.name):
                         raise DeviceError(
-                            "Observable {} not supported on device {}".format(
-                                i.name, self.short_name
-                            )
+                            f"Observable {i.name} not supported on device {self.short_name}"
                         )
             else:
                 observable_name = o.name
@@ -928,17 +1020,13 @@ class Device(abc.ABC):
                     ) or self.capabilities().get("inverse_operations", False)
                     if not supports_inv:
                         raise DeviceError(
-                            "The inverse of gates are not supported on device {}".format(
-                                self.short_name
-                            )
+                            f"The inverse of gates are not supported on device {self.short_name}"
                         )
                     observable_name = o.base_name
 
                 if not self.supports_observable(observable_name):
                     raise DeviceError(
-                        "Observable {} not supported on device {}".format(
-                            observable_name, self.short_name
-                        )
+                        f"Observable {observable_name} not supported on device {self.short_name}"
                     )
 
     @abc.abstractmethod
@@ -987,7 +1075,7 @@ class Device(abc.ABC):
             float: variance :math:`\mathrm{var}(A) = \bra{\psi}A^2\ket{\psi} - \bra{\psi}A\ket{\psi}^2`
         """
         raise NotImplementedError(
-            "Returning variances from QNodes not currently supported by {}".format(self.short_name)
+            f"Returning variances from QNodes not currently supported by {self.short_name}"
         )
 
     def sample(self, observable, wires, par):
@@ -1011,7 +1099,7 @@ class Device(abc.ABC):
             array[float]: samples in an array of dimension ``(shots,)``
         """
         raise NotImplementedError(
-            "Returning samples from QNodes not currently supported by {}".format(self.short_name)
+            f"Returning samples from QNodes not currently supported by {self.short_name}"
         )
 
     def probability(self, wires=None):
@@ -1029,7 +1117,7 @@ class Device(abc.ABC):
             state tuples are in lexicographical order.
         """
         raise NotImplementedError(
-            "Returning probability not currently supported by {}".format(self.short_name)
+            f"Returning probability not currently supported by {self.short_name}"
         )
 
     @abc.abstractmethod
