@@ -16,15 +16,14 @@ This module contains the CircuitGraph class which is used to generate a DAG (dir
 representation of a quantum circuit from an Operator queue.
 """
 # pylint: disable=too-many-branches,too-many-arguments,too-many-instance-attributes
-from collections import Counter, OrderedDict, namedtuple
+from collections import namedtuple
 
-import networkx as nx
-
-import pennylane as qml
+import retworkx as rx
 import numpy as np
 
+import pennylane as qml
+
 from pennylane.wires import Wires
-from .circuit_drawer import CircuitDrawer
 
 
 def _by_idx(x):
@@ -53,34 +52,6 @@ def _is_observable(x):
     return getattr(x, "return_type", None) is not None
 
 
-def _list_at_index_or_none(ls, idx):
-    """Return the element of a list at the given index if it exists, return None otherwise.
-
-    Args:
-        ls (list[object]): The target list
-        idx (int): The target index
-
-    Returns:
-        Union[object,NoneType]: The element at the target index or None
-    """
-    if len(ls) > idx:
-        return ls[idx]
-
-    return None
-
-
-def _is_returned_observable(op):
-    """Helper for the condition of having an observable or
-    measurement process in the return statement.
-
-    Returns:
-        bool: whether or not the observable or measurement process is in the
-        return statement
-    """
-    is_obs = isinstance(op, (qml.operation.Observable, qml.measure.MeasurementProcess))
-    return is_obs and op.return_type is not None
-
-
 Layer = namedtuple("Layer", ["ops", "param_inds"])
 """Parametrized layer of the circuit.
 
@@ -114,9 +85,8 @@ class CircuitGraph:
         ops (Iterable[.Operator]): quantum operators constituting the circuit, in temporal order
         obs (Iterable[.MeasurementProcess]): terminal measurements, in temporal order
         wires (.Wires): The addressable wire registers of the device that will be executing this graph
-        par_info (dict[int, dict[str, .Operation or int]]): Parameter information. Keys are
-            parameter indices (in the order they appear on the tape), and values are a
-            dictionary containing the corresponding operation and operation parameter index.
+        par_info (list[dict]): Parameter information. For each index, the entry is a dictionary containing an operation
+        and an index into that operation's parameters.
         trainable_params (set[int]): A set containing the indices of parameters that support
             differentiability. The indices provided match the order of appearence in the
             quantum circuit.
@@ -147,15 +117,15 @@ class CircuitGraph:
             op.queue_idx = k  # store the queue index in the Operator
 
             if hasattr(op, "return_type"):
-                if op.return_type is qml.operation.State:
+                if op.return_type is qml.measurements.State:
                     # State measurements contain no wires by default, but wires are
                     # required for the circuit drawer, so we recreate the state
                     # measurement with all wires
-                    op = qml.measure.MeasurementProcess(qml.operation.State, wires=wires)
+                    op = qml.measurements.MeasurementProcess(qml.measurements.State, wires=wires)
 
-                elif op.return_type is qml.operation.Sample and op.wires == Wires([]):
+                elif op.return_type is qml.measurements.Sample and op.wires == Wires([]):
                     # Sampling without specifying wires is treated as sampling all wires
-                    op = qml.measure.MeasurementProcess(qml.operation.Sample, wires=wires)
+                    op = qml.measurements.MeasurementProcess(qml.measurements.Sample, wires=wires)
 
                 op.queue_idx = k
 
@@ -167,22 +137,35 @@ class CircuitGraph:
 
         # TODO: State preparations demolish the incoming state entirely, and therefore should have no incoming edges.
 
-        self._graph = nx.DiGraph()  #: nx.DiGraph: DAG representation of the quantum circuit
+        self._graph = rx.PyDiGraph(
+            multigraph=False
+        )  #: rx.PyDiGraph: DAG representation of the quantum circuit
+
         # Iterate over each (populated) wire in the grid
         for wire in self._grid.values():
             # Add the first operator on the wire to the graph
             # This operator does not depend on any others
-            self._graph.add_node(wire[0])
+
+            # Check if wire[0] in self._grid.values()
+            # is already added to the graph; this
+            # condition avoids adding new nodes with
+            # the same value but different indexes
+            if wire[0] not in self._graph.nodes():
+                self._graph.add_node(wire[0])
 
             for i in range(1, len(wire)):
                 # For subsequent operators on the wire:
-                if wire[i] not in self._graph:
+                if wire[i] not in self._graph.nodes():
                     # Add them to the graph if they are not already
                     # in the graph (multi-qubit operators might already have been placed)
                     self._graph.add_node(wire[i])
 
                 # Create an edge between this and the previous operator
-                self._graph.add_edge(wire[i - 1], wire[i])
+                # There isn't any default value for the edge-data in
+                # rx.PyDiGraph.add_edge(); this is set to an empty string
+                self._graph.add_edge(
+                    self._graph.nodes().index(wire[i - 1]), self._graph.nodes().index(wire[i]), ""
+                )
 
         # For computing depth; want only a graph with the operations, not
         # including the observables
@@ -264,7 +247,7 @@ class CircuitGraph:
         Returns:
             list[Observable]: observables
         """
-        nodes = [node for node in self._graph.nodes if _is_observable(node)]
+        nodes = [node for node in self._graph.nodes() if _is_observable(node)]
         return sorted(nodes, key=_by_idx)
 
     @property
@@ -284,7 +267,7 @@ class CircuitGraph:
         Returns:
             list[Operation]: operations
         """
-        nodes = [node for node in self._graph.nodes if not _is_observable(node)]
+        nodes = [node for node in self._graph.nodes() if not _is_observable(node)]
         return sorted(nodes, key=_by_idx)
 
     @property
@@ -300,7 +283,7 @@ class CircuitGraph:
         and directed edges pointing from nodes to their immediate dependents/successors.
 
         Returns:
-            networkx.DiGraph: the directed acyclic graph representing the quantum circuit
+            retworkx.PyDiGraph: the directed acyclic graph representing the quantum circuit
         """
         return self._graph
 
@@ -324,7 +307,14 @@ class CircuitGraph:
         Returns:
             set[Operator]: ancestors of the given operators
         """
-        return set().union(*(nx.dag.ancestors(self._graph, o) for o in ops)) - set(ops)
+        anc = set(
+            self._graph.get_node_data(n)
+            for n in set().union(
+                # rx.ancestors() returns node indexes instead of node-values
+                *(rx.ancestors(self._graph, self._graph.nodes().index(o)) for o in ops)
+            )
+        )
+        return anc - set(ops)
 
     def descendants(self, ops):
         """Descendants of a given set of operators.
@@ -335,7 +325,14 @@ class CircuitGraph:
         Returns:
             set[Operator]: descendants of the given operators
         """
-        return set().union(*(nx.dag.descendants(self._graph, o) for o in ops)) - set(ops)
+        des = set(
+            self._graph.get_node_data(n)
+            for n in set().union(
+                # rx.descendants() returns node indexes instead of node-values
+                *(rx.descendants(self._graph, self._graph.nodes().index(o)) for o in ops)
+            )
+        )
+        return des - set(ops)
 
     def _in_topological_order(self, ops):
         """Sorts a set of operators in the circuit in a topological order.
@@ -346,8 +343,9 @@ class CircuitGraph:
         Returns:
             Iterable[Operator]: same set of operators, topologically ordered
         """
-        G = nx.DiGraph(self._graph.subgraph(ops))
-        return nx.dag.topological_sort(G)
+        G = self._graph.subgraph(list(self._graph.nodes().index(o) for o in ops))
+        indexes = rx.topological_sort(G)
+        return list(G[x] for x in indexes)
 
     def ancestors_in_order(self, ops):
         """Operator ancestors in a topological order.
@@ -360,8 +358,7 @@ class CircuitGraph:
         Returns:
             list[Operator]: ancestors of the given operators, topologically ordered
         """
-        # return self._in_topological_order(self.ancestors(ops))  # an abitrary topological order
-        return sorted(self.ancestors(ops), key=_by_idx)
+        return sorted(self.ancestors(ops), key=_by_idx)  # an abitrary topological order
 
     def descendants_in_order(self, ops):
         """Operator descendants in a topological order.
@@ -422,7 +419,7 @@ class CircuitGraph:
         current = Layer([], [])
         layers = [current]
 
-        for idx, info in self.par_info.items():
+        for idx, info in enumerate(self.par_info):
             if idx in self.trainable_params:
                 op = info["op"]
 
@@ -454,124 +451,6 @@ class CircuitGraph:
             post_queue = self.descendants_in_order(ops)
             yield LayerData(pre_queue, ops, tuple(param_inds), post_queue)
 
-    def greedy_layers(self, wire_order=None, show_all_wires=False):
-        """Greedily collected layers of the circuit. Empty slots are filled with ``None``.
-
-        Layers are built by pushing back gates in the circuit as far as possible, so that
-        every Gate is at the lower possible layer.
-
-        Args:
-            wire_order (Wires): the order (from top to bottom) to print the wires of the circuit
-            show_all_wires (bool): If True, all wires, including empty wires, are printed.
-
-        Returns:
-            Tuple[list[list[~.Operation]], list[list[~.Observable]]]:
-            Tuple of the circuits operations and the circuits observables, both indexed
-            by wires.
-        """
-        l = 0
-
-        operations = OrderedDict()
-        for key in sorted(self._grid):
-            operations[key] = self._grid[key]
-
-        for wire in operations:
-            operations[wire] = list(
-                filter(
-                    lambda op: not (
-                        isinstance(op, (qml.operation.Observable, qml.measure.MeasurementProcess))
-                        and op.return_type is not None
-                    ),
-                    operations[wire],
-                )
-            )
-
-        while True:
-            layer_ops = {wire: _list_at_index_or_none(operations[wire], l) for wire in operations}
-            num_ops = Counter(layer_ops.values())
-
-            if None in num_ops and num_ops[None] == len(operations):
-                break
-
-            for (wire, op) in layer_ops.items():
-                if op is None:
-                    operations[wire].append(None)
-                    continue
-
-                # push back to next layer if not all args wires are there yet
-                if len(op.wires) > num_ops[op]:
-                    operations[wire].insert(l, None)
-
-            l += 1
-
-        observables = OrderedDict()
-
-        if self.max_simultaneous_measurements == 1:
-
-            # There is a single measurement for every wire
-            for wire in sorted(self._grid):
-                observables[wire] = list(
-                    filter(
-                        lambda op: isinstance(
-                            op, (qml.operation.Observable, qml.measure.MeasurementProcess)
-                        )
-                        and op.return_type is not None,
-                        self._grid[wire],
-                    )
-                )
-                if not observables[wire]:
-                    observables[wire] = [None]
-        else:
-
-            # There are wire(s) with multiple measurements.
-            # We are creating a separate "visual block" at the end of the
-            # circuit for each observable and mapping observables with block
-            # indices.
-            num_observables = len(self.observables)
-            mp_map = dict(zip(self.observables, range(num_observables)))
-
-            for wire in sorted(self._grid):
-                # Initialize to None everywhere
-                observables[wire] = [None] * num_observables
-
-                for op in self._grid[wire]:
-                    if _is_returned_observable(op):
-                        obs_idx = mp_map[op]
-                        observables[wire][obs_idx] = op
-
-        if wire_order is not None:
-            temp_op_grid = OrderedDict()
-            temp_obs_grid = OrderedDict()
-
-            if show_all_wires:
-                permutation = [
-                    self.wires.labels.index(i) if i in self.wires else None
-                    for i in wire_order.labels
-                ]
-            else:
-                permutation = [
-                    self.wires.labels.index(i) for i in wire_order.labels if i in self.wires
-                ]
-
-            for i, j in enumerate(permutation):
-                if j is None:
-                    temp_op_grid[i] = [None] * len(operations[0])
-                    temp_obs_grid[i] = [None] * len(observables[0])
-                    continue
-
-                if j in operations:
-                    temp_op_grid[i] = operations[j]
-                if j in observables:
-                    temp_obs_grid[i] = observables[j]
-
-            operations = temp_op_grid
-            observables = temp_obs_grid
-
-        op_grid = [operations[wire] for wire in operations]
-        obs_grid = [observables[wire] for wire in observables]
-
-        return op_grid, obs_grid
-
     def update_node(self, old, new):
         """Replaces the given circuit graph node with a new one.
 
@@ -585,40 +464,13 @@ class CircuitGraph:
         # NOTE Does not alter the graph edges in any way. variable_deps is not changed, _grid is not changed. Dangerous!
         if new.wires != old.wires:
             raise ValueError("The new Operator must act on the same wires as the old one.")
+
         new.queue_idx = old.queue_idx
-        nx.relabel_nodes(self._graph, {old: new}, copy=False)  # change the graph in place
+        self._graph[self._graph.nodes().index(old)] = new
+
         self._operations = self.operations_in_order
         self._observables = self.observables_in_order
 
-    def draw(self, charset="unicode", wire_order=None, show_all_wires=False):
-        """Draw the CircuitGraph as a circuit diagram.
-
-        Args:
-            charset (str, optional): The charset that should be used. Currently, "unicode" and "ascii" are supported.
-            wire_order (Wires or None): the order (from top to bottom) to print the wires of the circuit
-            show_all_wires (bool): If True, all wires, including empty wires, are printed.
-
-        Raises:
-            ValueError: If the given charset is not supported
-
-        Returns:
-            str: The circuit diagram representation of the ``CircuitGraph``
-        """
-        if wire_order is not None:
-            wire_order = qml.wires.Wires.all_wires([wire_order, self.wires])
-
-        grid, obs = self.greedy_layers(wire_order=wire_order, show_all_wires=show_all_wires)
-
-        drawer = CircuitDrawer(
-            grid,
-            obs,
-            wires=wire_order or self.wires,
-            charset=charset,
-            show_all_wires=show_all_wires,
-        )
-
-        return drawer.draw()
-
     def get_depth(self):
         """Depth of the quantum circuit (longest path in the DAG)."""
         # If there are no operations in the circuit, the depth is 0
@@ -630,9 +482,10 @@ class CircuitGraph:
         # expressed in terms of edges, and we want it in terms of nodes).
         if self._depth is None and self.operations:
             if self._operation_graph is None:
-                self._operation_graph = self.graph.subgraph(self.operations)
-                self._depth = nx.dag_longest_path_length(self._operation_graph) + 1
-
+                self._operation_graph = self._graph.subgraph(
+                    list(self._graph.nodes().index(node) for node in self.operations)
+                )
+                self._depth = rx.dag_longest_path_length(self._operation_graph) + 1
         return self._depth
 
     def has_path(self, a, b):
@@ -645,7 +498,22 @@ class CircuitGraph:
         Returns:
             bool: returns ``True`` if a path exists
         """
-        return nx.has_path(self._graph, a, b)
+        if a == b:
+            return True
+
+        return (
+            len(
+                rx.digraph_dijkstra_shortest_paths(
+                    self._graph,
+                    self._graph.nodes().index(a),
+                    self._graph.nodes().index(b),
+                    weight_fn=None,
+                    default_weight=1.0,
+                    as_undirected=False,
+                )
+            )
+            != 0
+        )
 
     @property
     def max_simultaneous_measurements(self):
