@@ -29,6 +29,7 @@ import numpy as np
 from scipy.special import factorial as fac
 
 import pennylane as qml
+from pennylane.ops import Identity
 from pennylane import Device
 from .._version import __version__
 
@@ -511,7 +512,7 @@ def photon_number(cov, mu, params, hbar=2.0):
     """
     # pylint: disable=unused-argument
     ex = (np.trace(cov) + mu.T @ mu) / (2 * hbar) - 1 / 2
-    var = (np.trace(cov @ cov) + 2 * mu.T @ cov @ mu) / (2 * hbar ** 2) - 1 / 4
+    var = (np.trace(cov @ cov) + 2 * mu.T @ cov @ mu) / (2 * hbar**2) - 1 / 4
     return ex, var
 
 
@@ -571,7 +572,7 @@ def poly_quad_expectations(cov, mu, wires, device_wires, params, hbar=2.0):
 
     # HACK, we need access to the Poly instance in order to expand the matrix!
     # TODO: maybe we should make heisenberg_obs a class method or a static method to avoid this being a 'hack'?
-    op = qml.ops.PolyXP(Q, wires=wires)
+    op = qml.PolyXP(Q, wires=wires)
     Q = op.heisenberg_obs(device_wires)
 
     if Q.ndim == 1:
@@ -618,7 +619,7 @@ def fock_expectation(cov, mu, params, hbar=2.0):
     ex = fock_prob(cov, mu, params[0], hbar=hbar)
 
     # var[|n><n|] = E[|n><n|^2] -  E[|n><n|]^2 = E[|n><n|] -  E[|n><n|]^2
-    var = ex - ex ** 2
+    var = ex - ex**2
     return ex, var
 
 
@@ -655,6 +656,8 @@ class DefaultGaussian(Device):
     author = "Xanadu Inc."
 
     _operation_map = {
+        "Identity": Identity.identity_op,
+        "Snapshot": None,
         "Beamsplitter": beamsplitter,
         "ControlledAddition": controlled_addition,
         "ControlledPhase": controlled_phase,
@@ -687,6 +690,7 @@ class DefaultGaussian(Device):
         super().__init__(wires, shots, analytic=analytic)
         self.eng = None
         self.hbar = hbar
+        self._debugger = None
 
         self.reset()
 
@@ -699,7 +703,6 @@ class DefaultGaussian(Device):
             supports_finite_shots=True,
             returns_probs=False,
             returns_state=False,
-            supports_reversible_diff=False,
         )
         return capabilities
 
@@ -726,6 +729,12 @@ class DefaultGaussian(Device):
             self._state = self._operation_map[operation](*par, hbar=self.hbar)
             return  # we are done here
 
+        if operation == "Snapshot":
+            if self._debugger and self._debugger.active:
+                gaussian = {"cov_matrix": self._state[0].copy(), "means": self._state[1].copy()}
+                self._debugger.snapshots[len(self._debugger.snapshots)] = gaussian
+            return  # we are done here
+
         if "State" in operation:
             # set the new device state
             cov, mu = self._operation_map[operation](*par, hbar=self.hbar)
@@ -835,9 +844,7 @@ class DefaultGaussian(Device):
         elif observable == "QuadOperator":
             phi = par[0]
         else:
-            raise NotImplementedError(
-                "default.gaussian does not support sampling {}".format(observable)
-            )
+            raise NotImplementedError(f"default.gaussian does not support sampling {observable}")
 
         cov, mu = self.reduced_state(wires)
         rot = rotation(phi)
