@@ -23,11 +23,11 @@ import functools
 from string import ascii_letters as ABC
 
 import numpy as np
-from scipy.sparse import coo_matrix
+from scipy.sparse import csr_matrix
 
 import pennylane as qml
-from pennylane import QubitDevice, DeviceError, QubitStateVector, BasisState
-from pennylane.operation import DiagonalOperation
+from pennylane import QubitDevice, DeviceError, QubitStateVector, BasisState, Snapshot
+from pennylane.ops.qubit.attributes import diagonal_in_z_basis
 from pennylane.wires import WireError
 from .._version import __version__
 
@@ -79,10 +79,6 @@ class DefaultQubit(QubitDevice):
         shots (None, int): How many times the circuit should be evaluated (or sampled) to estimate
             the expectation values. Defaults to ``None`` if not specified, which means that the device
             returns analytical results.
-        cache (int): Number of device executions to store in a cache to speed up subsequent
-            executions. A value of ``0`` indicates that no caching will take place. Once filled,
-            older elements of the cache are removed and replaced with the most recent device
-            executions to keep the cache up to date.
     """
 
     name = "Default qubit PennyLane plugin"
@@ -92,11 +88,14 @@ class DefaultQubit(QubitDevice):
     author = "Xanadu Inc."
 
     operations = {
+        "Identity",
+        "Snapshot",
         "BasisState",
         "QubitStateVector",
         "QubitUnitary",
         "ControlledQubitUnitary",
         "MultiControlledX",
+        "IntegerComparator",
         "DiagonalQubitUnitary",
         "PauliX",
         "PauliY",
@@ -104,12 +103,18 @@ class DefaultQubit(QubitDevice):
         "MultiRZ",
         "Hadamard",
         "S",
+        "Adjoint(S)",
         "T",
+        "Adjoint(T)",
         "SX",
+        "Adjoint(SX)",
         "CNOT",
         "SWAP",
         "ISWAP",
+        "PSWAP",
+        "Adjoint(ISWAP)",
         "SISWAP",
+        "Adjoint(SISWAP)",
         "SQISW",
         "CSWAP",
         "Toffoli",
@@ -129,6 +134,7 @@ class DefaultQubit(QubitDevice):
         "IsingXX",
         "IsingYY",
         "IsingZZ",
+        "IsingXY",
         "SingleExcitation",
         "SingleExcitationPlus",
         "SingleExcitationMinus",
@@ -139,6 +145,7 @@ class DefaultQubit(QubitDevice):
         "QubitSum",
         "OrbitalRotation",
         "QFT",
+        "ECR",
     }
 
     observables = {
@@ -151,10 +158,17 @@ class DefaultQubit(QubitDevice):
         "Projector",
         "SparseHamiltonian",
         "Hamiltonian",
+        "Sum",
+        "SProd",
+        "Prod",
+        "Exp",
     }
 
-    def __init__(self, wires, *, shots=None, cache=0, analytic=None):
-        super().__init__(wires, shots, cache=cache, analytic=analytic)
+    def __init__(
+        self, wires, *, r_dtype=np.float64, c_dtype=np.complex128, shots=None, analytic=None
+    ):
+        super().__init__(wires, shots, r_dtype=r_dtype, c_dtype=c_dtype, analytic=analytic)
+        self._debugger = None
 
         # Create the initial state. Internally, we store the
         # state as an array of dimension [2]*wires.
@@ -175,6 +189,21 @@ class DefaultQubit(QubitDevice):
             "Toffoli": self._apply_toffoli,
         }
 
+    @property
+    def stopping_condition(self):
+        def accepts_obj(obj):
+            if obj.name == "QFT" and len(obj.wires) >= 6:
+                return False
+            if obj.name == "GroverOperator" and len(obj.wires) >= 13:
+                return False
+            if getattr(obj, "has_matrix", False):
+                # pow operations dont work with backprop or adjoint without decomposition
+                # use class name string so we don't need to use isinstance check
+                return not (obj.__class__.__name__ == "Pow" and qml.operation.is_trainable(obj))
+            return obj.name in self.observables.union(self.operations)
+
+        return qml.BooleanFn(accepts_obj)
+
     @functools.lru_cache()
     def map_wires(self, wires):
         # temporarily overwrite this method to bypass
@@ -183,9 +212,7 @@ class DefaultQubit(QubitDevice):
             mapped_wires = [self.wire_map[w] for w in wires]
         except KeyError as e:
             raise WireError(
-                "Did not find some of the wires {} on device with wires {}.".format(
-                    wires.labels, self.wires.labels
-                )
+                f"Did not find some of the wires {wires.labels} on device with wires {self.wires.labels}."
             ) from e
 
         return mapped_wires
@@ -197,6 +224,17 @@ class DefaultQubit(QubitDevice):
         wire_map = zip(wires, consecutive_wires)
         return dict(wire_map)
 
+    # pylint: disable=arguments-differ
+    def _get_batch_size(self, tensor, expected_shape, expected_size):
+        """Determine whether a tensor has an additional batch dimension for broadcasting,
+        compared to an expected_shape."""
+        size = self._size(tensor)
+        if self._ndim(tensor) > len(expected_shape) or size > expected_size:
+            return size // expected_size
+
+        return None
+
+    # pylint: disable=arguments-differ
     def apply(self, operations, rotations=None, **kwargs):
         rotations = rotations or []
 
@@ -205,14 +243,21 @@ class DefaultQubit(QubitDevice):
 
             if i > 0 and isinstance(operation, (QubitStateVector, BasisState)):
                 raise DeviceError(
-                    "Operation {} cannot be used after other Operations have already been applied "
-                    "on a {} device.".format(operation.name, self.short_name)
+                    f"Operation {operation.name} cannot be used after other Operations have already been applied "
+                    f"on a {self.short_name} device."
                 )
 
             if isinstance(operation, QubitStateVector):
                 self._apply_state_vector(operation.parameters[0], operation.wires)
             elif isinstance(operation, BasisState):
                 self._apply_basis_state(operation.parameters[0], operation.wires)
+            elif isinstance(operation, Snapshot):
+                if self._debugger and self._debugger.active:
+                    state_vector = np.array(self._flatten(self._state))
+                    if operation.tag:
+                        self._debugger.snapshots[operation.tag] = state_vector
+                    else:
+                        self._debugger.snapshots[len(self._debugger.snapshots)] = state_vector
             else:
                 self._state = self._apply_operation(self._state, operation)
 
@@ -233,15 +278,18 @@ class DefaultQubit(QubitDevice):
         Returns:
             array[complex]: output state
         """
+        if operation.__class__.__name__ == "Identity":
+            return state
         wires = operation.wires
 
-        if operation.base_name in self._apply_ops:
-            axes = self.wires.indices(wires)
+        if operation.__class__.__name__ in self._apply_ops:
+            shift = int(self._ndim(state) > self.num_wires)
+            axes = [ax + shift for ax in self.wires.indices(wires)]
             return self._apply_ops[operation.base_name](state, axes, inverse=operation.inverse)
 
-        matrix = self._get_unitary_matrix(operation)
+        matrix = self._asarray(self._get_unitary_matrix(operation), dtype=self.C_DTYPE)
 
-        if isinstance(operation, DiagonalOperation):
+        if operation in diagonal_in_z_basis:
             return self._apply_diagonal_unitary(state, matrix, wires)
         if len(wires) <= 2:
             # Einsum is faster for small gates
@@ -304,7 +352,7 @@ class DefaultQubit(QubitDevice):
         """
         state_x = self._apply_x(state, axes)
         state_z = self._apply_z(state, axes)
-        return SQRT2INV * (state_x + state_z)
+        return self._const_mul(SQRT2INV, state_x + state_z)
 
     def _apply_s(self, state, axes, inverse=False):
         return self._apply_phase(state, axes, 1j, inverse)
@@ -342,8 +390,9 @@ class DefaultQubit(QubitDevice):
         Returns:
             array[complex]: output state
         """
-        sl_0 = _get_slice(0, axes[0], self.num_wires)
-        sl_1 = _get_slice(1, axes[0], self.num_wires)
+        ndim = self._ndim(state)
+        sl_0 = _get_slice(0, axes[0], ndim)
+        sl_1 = _get_slice(1, axes[0], ndim)
 
         # We will be slicing into the state according to state[sl_1], giving us all of the
         # amplitudes with a |1> for the control qubit. The resulting array has lost an axis
@@ -377,10 +426,11 @@ class DefaultQubit(QubitDevice):
         """
         cntrl_max = np.argmax(axes[:2])
         cntrl_min = cntrl_max ^ 1
-        sl_a0 = _get_slice(0, axes[cntrl_max], self.num_wires)
-        sl_a1 = _get_slice(1, axes[cntrl_max], self.num_wires)
-        sl_b0 = _get_slice(0, axes[cntrl_min], self.num_wires - 1)
-        sl_b1 = _get_slice(1, axes[cntrl_min], self.num_wires - 1)
+        ndim = self._ndim(state)
+        sl_a0 = _get_slice(0, axes[cntrl_max], ndim)
+        sl_a1 = _get_slice(1, axes[cntrl_max], ndim)
+        sl_b0 = _get_slice(0, axes[cntrl_min], ndim - 1)
+        sl_b1 = _get_slice(1, axes[cntrl_min], ndim - 1)
 
         # If both controls are smaller than the target, shift the target axis down by two. If one
         # control is greater and one control is smaller than the target, shift the target axis
@@ -423,8 +473,9 @@ class DefaultQubit(QubitDevice):
         Returns:
             array[complex]: output state
         """
-        sl_0 = _get_slice(0, axes[0], self.num_wires)
-        sl_1 = _get_slice(1, axes[0], self.num_wires)
+        ndim = self._ndim(state)
+        sl_0 = _get_slice(0, axes[0], ndim)
+        sl_1 = _get_slice(1, axes[0], ndim)
 
         if axes[1] > axes[0]:
             target_axes = [axes[1] - 1]
@@ -446,17 +497,17 @@ class DefaultQubit(QubitDevice):
         Returns:
             array[complex]: output state
         """
-        num_wires = len(state.shape)
-        sl_0 = _get_slice(0, axes[0], num_wires)
-        sl_1 = _get_slice(1, axes[0], num_wires)
+        ndim = self._ndim(state)
+        sl_0 = _get_slice(0, axes[0], ndim)
+        sl_1 = _get_slice(1, axes[0], ndim)
 
         phase = self._conj(parameters) if inverse else parameters
-        return self._stack([state[sl_0], phase * state[sl_1]], axis=axes[0])
+        return self._stack([state[sl_0], self._const_mul(phase, state[sl_1])], axis=axes[0])
 
     def expval(self, observable, shot_range=None, bin_size=None):
         """Returns the expectation value of a Hamiltonian observable. When the observable is a
-         ``SparseHamiltonian`` object, the expectation value is computed directly for the full
-         Hamiltonian, which leads to faster execution.
+        ``Hamiltonian`` or ``SparseHamiltonian`` object, the expectation value is computed directly
+        from the sparse matrix representation, which leads to faster execution.
 
         Args:
             observable (~.Observable): a PennyLane observable
@@ -468,6 +519,13 @@ class DefaultQubit(QubitDevice):
 
         Returns:
             float: returns the expectation value of the observable
+
+        .. warning::
+
+            This function does not support broadcasting if ``observable`` is a
+            :class:``~.Hamiltonian`` and the device interface or the interface of the
+            Hamiltonian is not NumPy or Autograd
+
         """
         # intercept other Hamiltonians
         # TODO: Ideally, this logic should not live in the Device, but be moved
@@ -475,12 +533,19 @@ class DefaultQubit(QubitDevice):
         if observable.name in ("Hamiltonian", "SparseHamiltonian"):
             assert self.shots is None, f"{observable.name} must be used with shots=None"
 
+            self.map_wires(observable.wires)
             backprop_mode = (
                 not isinstance(self.state, np.ndarray)
                 or any(not isinstance(d, (float, np.ndarray)) for d in observable.data)
             ) and observable.name == "Hamiltonian"
 
             if backprop_mode:
+                # TODO[dwierichs]: This branch is not adapted to broadcasting yet
+                if self._ndim(self.state) == 2:
+                    raise NotImplementedError(
+                        "Expectation values of Hamiltonians for interface!=None are "
+                        "not supported together with parameter broadcasting yet"
+                    )
                 # We must compute the expectation value assuming that the Hamiltonian
                 # coefficients *and* the quantum states are tensor objects.
 
@@ -489,19 +554,20 @@ class DefaultQubit(QubitDevice):
                 res = qml.math.cast(qml.math.convert_like(0.0, observable.data), dtype=complex)
                 interface = qml.math.get_interface(self.state)
 
-                # Note: it is important that we use the Hamiltonian's data and not the coeffs attribute.
-                # This is because the .data attribute may be 'unwrapped' as required by the interfaces,
-                # whereas the .coeff attribute will always be the same input dtype that the user provided.
+                # Note: it is important that we use the Hamiltonian's data and not the coeffs
+                # attribute. This is because the .data attribute may be 'unwrapped' as required by
+                # the interfaces, whereas the .coeff attribute will always be the same input dtype
+                # that the user provided.
                 for op, coeff in zip(observable.ops, observable.data):
 
                     # extract a scipy.sparse.coo_matrix representation of this Pauli word
-                    coo = qml.operation.Tensor(op).sparse_matrix(wires=self.wires)
-                    Hmat = qml.math.cast(qml.math.convert_like(coo.data, self.state), "complex128")
+                    coo = qml.operation.Tensor(op).sparse_matrix(wires=self.wires, format="coo")
+                    Hmat = qml.math.cast(qml.math.convert_like(coo.data, self.state), self.C_DTYPE)
 
                     product = (
-                        qml.math.gather(qml.math.conj(self.state), coo.row)
+                        self._gather(self._conj(self.state), coo.row)
                         * Hmat
-                        * qml.math.gather(self.state, coo.col)
+                        * self._gather(self.state, coo.col)
                     )
                     c = qml.math.convert_like(coeff, product)
 
@@ -513,22 +579,32 @@ class DefaultQubit(QubitDevice):
             else:
                 # Coefficients and the state are not trainable, we can be more
                 # efficient in how we compute the Hamiltonian sparse matrix.
-
                 if observable.name == "Hamiltonian":
                     Hmat = qml.utils.sparse_hamiltonian(observable, wires=self.wires)
                 elif observable.name == "SparseHamiltonian":
-                    Hmat = observable.matrix
+                    Hmat = observable.sparse_matrix()
 
                 state = qml.math.toarray(self.state)
-                res = coo_matrix.dot(
-                    coo_matrix(qml.math.conj(state)),
-                    coo_matrix.dot(Hmat, coo_matrix(state.reshape(len(self.state), 1))),
-                ).toarray()[0]
+                if self._ndim(state) == 2:
+                    res = qml.math.array(
+                        [
+                            csr_matrix.dot(
+                                csr_matrix(self._conj(_state)),
+                                csr_matrix.dot(Hmat, csr_matrix(_state[..., None])),
+                            ).toarray()[0]
+                            for _state in state
+                        ]
+                    )
+                else:
+                    res = csr_matrix.dot(
+                        csr_matrix(self._conj(state)),
+                        csr_matrix.dot(Hmat, csr_matrix(state[..., None])),
+                    ).toarray()[0]
 
             if observable.name == "Hamiltonian":
                 res = qml.math.squeeze(res)
 
-            return qml.math.real(res)
+            return self._real(res)
 
         return super().expval(observable, shot_range=shot_range, bin_size=bin_size)
 
@@ -543,19 +619,19 @@ class DefaultQubit(QubitDevice):
             the unitary in the computational basis, or, in the case of a diagonal unitary,
             a 1D array representing the matrix diagonal.
         """
-        if isinstance(unitary, DiagonalOperation):
-            return unitary.eigvals
+        if unitary in diagonal_in_z_basis:
+            return unitary.eigvals()
 
-        return unitary.matrix
+        return unitary.matrix()
 
     @classmethod
     def capabilities(cls):
         capabilities = super().capabilities().copy()
         capabilities.update(
             model="qubit",
-            supports_reversible_diff=True,
             supports_inverse_operations=True,
             supports_analytic_computation=True,
+            supports_broadcasting=True,
             returns_state=True,
             passthru_devices={
                 "tf": "default.qubit.tf",
@@ -575,72 +651,52 @@ class DefaultQubit(QubitDevice):
         Returns:
             array[complex]: complex array of shape ``[2]*self.num_wires``
             representing the statevector of the basis state
+
+        Note: This function does not support broadcasted inputs yet.
         """
-        state = np.zeros(2 ** self.num_wires, dtype=np.complex128)
+        state = np.zeros(2**self.num_wires, dtype=np.complex128)
         state[index] = 1
         state = self._asarray(state, dtype=self.C_DTYPE)
         return self._reshape(state, [2] * self.num_wires)
 
     @property
     def state(self):
-        return self._flatten(self._pre_rotated_state)
-
-    def density_matrix(self, wires):
-        """Returns the reduced density matrix of a given set of wires.
-
-        Args:
-            wires (Wires): wires of the reduced system.
-
-        Returns:
-            array[complex]: complex tensor of shape ``(2 ** len(wires), 2 ** len(wires))``
-            representing the reduced density matrix.
-        """
-        dim = self.num_wires
-        state = self._pre_rotated_state
-
-        # Return the full density matrix by using numpy tensor product
-        if wires == self.wires:
-            density_matrix = self._tensordot(state, self._conj(state), 0)
-            density_matrix = self._reshape(density_matrix, (2 ** len(wires), 2 ** len(wires)))
-            return density_matrix
-
-        complete_system = list(range(0, dim))
-        traced_system = [x for x in complete_system if x not in wires.labels]
-
-        # Return the reduced density matrix by using numpy tensor product
-        density_matrix = self._tensordot(
-            state, self._conj(state), axes=(traced_system, traced_system)
-        )
-        density_matrix = self._reshape(density_matrix, (2 ** len(wires), 2 ** len(wires)))
-
-        return density_matrix
+        dim = 2**self.num_wires
+        batch_size = self._get_batch_size(self._pre_rotated_state, (2,) * self.num_wires, dim)
+        # Do not flatten the state completely but leave the broadcasting dimension if there is one
+        shape = (batch_size, dim) if batch_size is not None else (dim,)
+        return self._reshape(self._pre_rotated_state, shape)
 
     def _apply_state_vector(self, state, device_wires):
         """Initialize the internal state vector in a specified state.
 
         Args:
-            state (array[complex]): normalized input state of length
-                ``2**len(wires)``
+            state (array[complex]): normalized input state of length ``2**len(wires)``
+                or broadcasted state of shape ``(batch_size, 2**len(wires))``
             device_wires (Wires): wires that get initialized in the state
         """
 
         # translate to wire labels used by device
         device_wires = self.map_wires(device_wires)
+        dim = 2 ** len(device_wires)
 
         state = self._asarray(state, dtype=self.C_DTYPE)
-        n_state_vector = state.shape[0]
+        batch_size = self._get_batch_size(state, (dim,), dim)
+        output_shape = [2] * self.num_wires
+        if batch_size is not None:
+            output_shape.insert(0, batch_size)
 
-        if len(qml.math.shape(state)) != 1 or n_state_vector != 2 ** len(device_wires):
-            raise ValueError("State vector must be of length 2**wires.")
+        if not (state.shape in [(dim,), (batch_size, dim)]):
+            raise ValueError("State vector must have shape (2**wires,) or (batch_size, 2**wires).")
 
-        norm = qml.math.linalg.norm(state, ord=2)
-        if not qml.math.is_abstract(norm):
+        if not qml.math.is_abstract(state):
+            norm = qml.math.linalg.norm(state, axis=-1, ord=2)
             if not qml.math.allclose(norm, 1.0, atol=tolerance):
                 raise ValueError("Sum of amplitudes-squared does not equal one.")
 
         if len(device_wires) == self.num_wires and sorted(device_wires) == device_wires:
-            # Initialize the entire wires with the state
-            self._state = self._reshape(state, [2] * self.num_wires)
+            # Initialize the entire device state with the input state
+            self._state = self._reshape(state, output_shape)
             return
 
         # generate basis states on subset of qubits via the cartesian product
@@ -653,8 +709,13 @@ class DefaultQubit(QubitDevice):
         # get indices for which the state is changed to input state vector elements
         ravelled_indices = np.ravel_multi_index(unravelled_indices.T, [2] * self.num_wires)
 
-        state = self._scatter(ravelled_indices, state, [2 ** self.num_wires])
-        state = self._reshape(state, [2] * self.num_wires)
+        if batch_size is not None:
+            state = self._scatter(
+                (slice(None), ravelled_indices), state, [batch_size, 2**self.num_wires]
+            )
+        else:
+            state = self._scatter(ravelled_indices, state, [2**self.num_wires])
+        state = self._reshape(state, output_shape)
         self._state = self._asarray(state, dtype=self.C_DTYPE)
 
     def _apply_basis_state(self, state, wires):
@@ -664,6 +725,8 @@ class DefaultQubit(QubitDevice):
             state (array[int]): computational basis state of shape ``(wires,)``
                 consisting of 0s and 1s.
             wires (Wires): wires that the provided computational state should be initialized on
+
+        Note: This function does not support broadcasted inputs yet.
         """
         # translate to wire labels used by device
         device_wires = self.map_wires(wires)
@@ -694,12 +757,32 @@ class DefaultQubit(QubitDevice):
 
         Returns:
             array[complex]: output state
+
+        Note: This function does not support simultaneously broadcasted states and matrices yet.
         """
         # translate to wire labels used by device
         device_wires = self.map_wires(wires)
 
-        mat = self._cast(self._reshape(mat, [2] * len(device_wires) * 2), dtype=self.C_DTYPE)
-        axes = (np.arange(len(device_wires), 2 * len(device_wires)), device_wires)
+        dim = 2 ** len(device_wires)
+        mat_batch_size = self._get_batch_size(mat, (dim, dim), dim**2)
+        state_batch_size = self._get_batch_size(state, (2,) * self.num_wires, 2**self.num_wires)
+
+        shape = [2] * (len(device_wires) * 2)
+        state_axes = device_wires
+        # If the matrix is broadcasted, it is reshaped to have leading axis of size mat_batch_size
+        if mat_batch_size:
+            shape.insert(0, mat_batch_size)
+            if state_batch_size:
+                raise NotImplementedError(
+                    "Applying a broadcasted unitary to an already broadcasted state via "
+                    "_apply_unitary is not supported. Broadcasting sizes are "
+                    f"({mat_batch_size}, {state_batch_size})."
+                )
+        # If the state is broadcasted, the affected state axes need to be shifted by 1.
+        if state_batch_size:
+            state_axes = [ax + 1 for ax in state_axes]
+        mat = self._cast(self._reshape(mat, shape), dtype=self.C_DTYPE)
+        axes = (np.arange(-len(device_wires), 0), state_axes)
         tdot = self._tensordot(mat, state, axes=axes)
 
         # tensordot causes the axes given in `wires` to end up in the first positions
@@ -708,6 +791,16 @@ class DefaultQubit(QubitDevice):
         # We'll need to invert this permutation to put the indices in the correct place
         unused_idxs = [idx for idx in range(self.num_wires) if idx not in device_wires]
         perm = list(device_wires) + unused_idxs
+        # If the matrix is broadcasted, all but the first dimension are shifted by 1
+        if mat_batch_size:
+            perm = [idx + 1 for idx in perm]
+            perm.insert(0, 0)
+        if state_batch_size:
+            # As the state broadcasting dimension always is the first in the state, it always
+            # ends up in position `len(device_wires)` after the tensordot. The -1 causes it
+            # being permuted to the leading dimension after transposition
+            perm.insert(len(device_wires), -1)
+
         inv_perm = np.argsort(perm)  # argsort gives inverse permutation
         return self._transpose(tdot, inv_perm)
 
@@ -728,7 +821,14 @@ class DefaultQubit(QubitDevice):
         # translate to wire labels used by device
         device_wires = self.map_wires(wires)
 
-        mat = self._cast(self._reshape(mat, [2] * len(device_wires) * 2), dtype=self.C_DTYPE)
+        dim = 2 ** len(device_wires)
+        batch_size = self._get_batch_size(mat, (dim, dim), dim**2)
+
+        # If the matrix is broadcasted, it is reshaped to have leading axis of size mat_batch_size
+        shape = [2] * (len(device_wires) * 2)
+        if batch_size is not None:
+            shape.insert(0, batch_size)
+        mat = self._cast(self._reshape(mat, shape), dtype=self.C_DTYPE)
 
         # Tensor indices of the quantum state
         state_indices = ABC[: self.num_wires]
@@ -748,13 +848,9 @@ class DefaultQubit(QubitDevice):
         )
 
         # We now put together the indices in the notation numpy's einsum requires
+        # This notation allows for the state, the matrix, or both to be broadcasted
         einsum_indices = (
-            "{new_indices}{affected_indices},{state_indices}->{new_state_indices}".format(
-                affected_indices=affected_indices,
-                state_indices=state_indices,
-                new_indices=new_indices,
-                new_state_indices=new_state_indices,
-            )
+            f"...{new_indices}{affected_indices},...{state_indices}->...{new_state_indices}"
         )
 
         return self._einsum(einsum_indices, mat, state)
@@ -774,17 +870,19 @@ class DefaultQubit(QubitDevice):
         """
         # translate to wire labels used by device
         device_wires = self.map_wires(wires)
+        dim = 2 ** len(device_wires)
+        batch_size = self._get_batch_size(phases, (dim,), dim)
 
         # reshape vectors
-        phases = self._cast(self._reshape(phases, [2] * len(device_wires)), dtype=self.C_DTYPE)
+        shape = [2] * len(device_wires)
+        if batch_size is not None:
+            shape.insert(0, batch_size)
+        phases = self._cast(self._reshape(phases, shape), dtype=self.C_DTYPE)
 
         state_indices = ABC[: self.num_wires]
         affected_indices = "".join(ABC_ARRAY[list(device_wires)].tolist())
 
-        einsum_indices = "{affected_indices},{state_indices}->{state_indices}".format(
-            affected_indices=affected_indices, state_indices=state_indices
-        )
-
+        einsum_indices = f"...{affected_indices},...{state_indices}->...{state_indices}"
         return self._einsum(einsum_indices, phases, state)
 
     def reset(self):
@@ -800,5 +898,136 @@ class DefaultQubit(QubitDevice):
         if self._state is None:
             return None
 
-        prob = self.marginal_prob(self._abs(self._flatten(self._state)) ** 2, wires)
-        return prob
+        dim = 2**self.num_wires
+        batch_size = self._get_batch_size(self._state, [2] * self.num_wires, dim)
+        flat_state = self._reshape(
+            self._state, (batch_size, dim) if batch_size is not None else (dim,)
+        )
+        real_state = self._real(flat_state)
+        imag_state = self._imag(flat_state)
+        return self.marginal_prob(real_state**2 + imag_state**2, wires)
+
+    def classical_shadow(self, obs, circuit):
+        """
+        Returns the measured bits and recipes in the classical shadow protocol.
+
+        The protocol is described in detail in the `classical shadows paper <https://arxiv.org/abs/2002.08953>`_.
+        This measurement process returns the randomized Pauli measurements (the ``recipes``)
+        that are performed for each qubit and snapshot as an integer:
+
+        - 0 for Pauli X,
+        - 1 for Pauli Y, and
+        - 2 for Pauli Z.
+
+        It also returns the measurement results (the ``bits``); 0 if the 1 eigenvalue
+        is sampled, and 1 if the -1 eigenvalue is sampled.
+
+        The device shots are used to specify the number of snapshots. If ``T`` is the number
+        of shots and ``n`` is the number of qubits, then both the measured bits and the
+        Pauli measurements have shape ``(T, n)``.
+
+        This implementation leverages vectorization and offers a significant speed-up over
+        the generic implementation.
+
+        .. Note::
+
+            This method internally calls ``np.einsum`` which supports at most 52 indices,
+            thus the classical shadow measurement for this device supports at most 52
+            qubits.
+
+        .. seealso:: :func:`~.classical_shadow`
+
+        Args:
+            obs (~.pennylane.measurements.ShadowMeasurementProcess): The classical shadow measurement process
+            circuit (~.tapes.QuantumTape): The quantum tape that is being executed
+
+        Returns:
+            tensor_like[int]: A tensor with shape ``(2, T, n)``, where the first row represents
+            the measured bits and the second represents the recipes used.
+        """
+        wires = obs.wires
+        seed = obs.seed
+
+        n_qubits = len(wires)
+        n_snapshots = self.shots
+        device_qubits = len(self.wires)
+        mapped_wires = np.array(self.map_wires(wires))
+
+        if seed is not None:
+            # seed the random measurement generation so that recipes
+            # are the same for different executions with the same seed
+            rng = np.random.RandomState(seed)
+            recipes = rng.randint(0, 3, size=(n_snapshots, n_qubits))
+        else:
+            recipes = np.random.randint(0, 3, size=(n_snapshots, n_qubits))
+
+        obs_list = self._stack(
+            [
+                qml.PauliX.compute_matrix(),
+                qml.PauliY.compute_matrix(),
+                qml.PauliZ.compute_matrix(),
+            ]
+        )
+        uni_list = self._stack(
+            [
+                qml.Hadamard.compute_matrix(),
+                qml.Hadamard.compute_matrix() @ qml.RZ.compute_matrix(-np.pi / 2),
+                qml.Identity.compute_matrix(),
+            ]
+        )
+        obs = obs_list[recipes]
+        uni = uni_list[recipes]
+
+        # There's a significant speedup if we use the following iterative
+        # process to perform the randomized Pauli measurements:
+        #   1. Randomly generate Pauli observables for all snapshots for
+        #      a single qubit (e.g. the first qubit).
+        #   2. Compute the expectation of each Pauli observable on the first
+        #      qubit by tracing out all other qubits.
+        #   3. Sample the first qubit based on each Pauli expectation.
+        #   4. For all snapshots, determine the collapsed state of the remaining
+        #      qubits based on the sample result.
+        #   4. Repeat iteratively until no qubits are remaining.
+        #
+        # Observe that after the first iteration, the second qubit will become the
+        # "first" qubit in the process. The advantage to this approach as opposed to
+        # simulataneously computing the Pauli expectations for each qubit is that
+        # the partial traces are computed over iteratively smaller subsystems, leading
+        # to a significant speed-up.
+
+        # transpose the state so that the measured wires appear first
+        unmeasured_wires = [i for i in range(len(self.wires)) if i not in mapped_wires]
+        transposed_state = np.transpose(self._state, axes=mapped_wires.tolist() + unmeasured_wires)
+
+        outcomes = np.zeros((n_snapshots, n_qubits))
+        stacked_state = self._stack([transposed_state for _ in range(n_snapshots)])
+
+        for i in range(n_qubits):
+
+            # trace out every qubit except the first
+            first_qubit_state = self._einsum(
+                f"{ABC[device_qubits - i + 1]}{ABC[:device_qubits - i]},{ABC[device_qubits - i + 1]}{ABC[device_qubits - i]}{ABC[1:device_qubits - i]}"
+                f"->{ABC[device_qubits - i + 1]}a{ABC[device_qubits - i]}",
+                stacked_state,
+                self._conj(stacked_state),
+            )
+
+            # sample the observables on the first qubit
+            probs = (self._einsum("abc,acb->a", first_qubit_state, obs[:, i]) + 1) / 2
+            samples = np.random.uniform(0, 1, size=probs.shape) > probs
+            outcomes[:, i] = samples
+
+            # collapse the state of the remaining qubits; the next qubit in line
+            # becomes the first qubit for the next iteration
+            rotated_state = self._einsum("ab...,acb->ac...", stacked_state, uni[:, i])
+            stacked_state = rotated_state[np.arange(n_snapshots), self._cast(samples, np.int8)]
+
+            # re-normalize the collapsed state
+            norms = np.sqrt(
+                np.sum(
+                    np.abs(stacked_state) ** 2, tuple(range(1, device_qubits - i)), keepdims=True
+                )
+            )
+            stacked_state /= norms
+
+        return self._cast(self._stack([outcomes, recipes]), dtype=np.int8)
