@@ -23,11 +23,11 @@ from math import cos, sin, sqrt
 
 import pytest
 import numpy as np
-import pennylane as qml
-
 from scipy.linalg import block_diag
 from flaky import flaky
 
+import pennylane as qml
+
 pytestmark = pytest.mark.skip_unsupported
 
 np.random.seed(42)
@@ -37,6 +37,8 @@ np.random.seed(42)
 
 # gates for which device support is tested
 ops = {
+    "Identity": qml.Identity(wires=[0]),
+    "Snapshot": qml.Snapshot("label"),
     "BasisState": qml.BasisState(np.array([0]), wires=[0]),
     "CNOT": qml.CNOT(wires=[0, 1]),
     "CRX": qml.CRX(0, wires=[0, 1]),
@@ -55,23 +57,34 @@ ops = {
     "PhaseShift": qml.PhaseShift(0, wires=[0]),
     "ControlledPhaseShift": qml.ControlledPhaseShift(0, wires=[0, 1]),
     "QubitStateVector": qml.QubitStateVector(np.array([1.0, 0.0]), wires=[0]),
+    "QubitDensityMatrix": qml.QubitDensityMatrix(np.array([[0.5, 0.0], [0, 0.5]]), wires=[0]),
     "QubitUnitary": qml.QubitUnitary(np.eye(2), wires=[0]),
     "ControlledQubitUnitary": qml.ControlledQubitUnitary(np.eye(2), control_wires=[1], wires=[0]),
     "MultiControlledX": qml.MultiControlledX(control_wires=[1, 2], wires=[0]),
+    "IntegerComparator": qml.IntegerComparator(1, geq=True, wires=[0, 1, 2]),
     "RX": qml.RX(0, wires=[0]),
     "RY": qml.RY(0, wires=[0]),
     "RZ": qml.RZ(0, wires=[0]),
     "Rot": qml.Rot(0, 0, 0, wires=[0]),
     "S": qml.S(wires=[0]),
+    "Adjoint(S)": qml.adjoint(qml.S(wires=[0])),
     "SWAP": qml.SWAP(wires=[0, 1]),
     "ISWAP": qml.ISWAP(wires=[0, 1]),
+    "PSWAP": qml.PSWAP(0, wires=[0, 1]),
+    "ECR": qml.ECR(wires=[0, 1]),
+    "Adjoint(ISWAP)": qml.adjoint(qml.ISWAP(wires=[0, 1])),
     "T": qml.T(wires=[0]),
+    "Adjoint(T)": qml.adjoint(qml.T(wires=[0])),
     "SX": qml.SX(wires=[0]),
+    "Adjoint(SX)": qml.adjoint(qml.SX(wires=[0])),
+    "Barrier": qml.Barrier(wires=[0, 1, 2]),
+    "WireCut": qml.WireCut(wires=[0]),
     "Toffoli": qml.Toffoli(wires=[0, 1, 2]),
     "QFT": qml.templates.QFT(wires=[0, 1, 2]),
     "IsingXX": qml.IsingXX(0, wires=[0, 1]),
     "IsingYY": qml.IsingYY(0, wires=[0, 1]),
     "IsingZZ": qml.IsingZZ(0, wires=[0, 1]),
+    "IsingXY": qml.IsingXY(0, wires=[0, 1]),
     "SingleExcitation": qml.SingleExcitation(0, wires=[0, 1]),
     "SingleExcitationPlus": qml.SingleExcitationPlus(0, wires=[0, 1]),
     "SingleExcitationMinus": qml.SingleExcitationMinus(0, wires=[0, 1]),
@@ -85,18 +98,22 @@ ops = {
     "U2": qml.U2(0, 0, wires=0),
     "U3": qml.U3(0, 0, 0, wires=0),
     "SISWAP": qml.SISWAP(wires=[0, 1]),
+    "Adjoint(SISWAP)": qml.adjoint(qml.SISWAP(wires=[0, 1])),
     "OrbitalRotation": qml.OrbitalRotation(0, wires=[0, 1, 2, 3]),
 }
 
 all_ops = ops.keys()
 
 # All qubit operations should be available to test in the device test suite
-all_available_ops = qml.ops._qubit__ops__.copy()  # pylint: disable=protected-access
+# Linting check disabled as static analysis can misidentify qml.ops as the set instance qml.ops.qubit.ops
+all_available_ops = qml.ops._qubit__ops__.copy()  # pylint: disable=protected-access,no-member
 all_available_ops.remove("CPhase")  # CPhase is an alias of ControlledPhaseShift
 all_available_ops.remove("SQISW")  # SQISW is an alias of SISWAP
 all_available_ops.add("QFT")  # QFT was recently moved to being a template, but let's keep it here
 
-if not set(all_ops) == all_available_ops:
+symbolic_ops = {"Adjoint(S)", "Adjoint(T)", "Adjoint(SX)", "Adjoint(ISWAP)", "Adjoint(SISWAP)"}
+
+if not set(all_ops) == all_available_ops.union(symbolic_ops):
     raise ValueError(
         "A qubit operation has been added that is not being tested in the "
         "device test suite. Please add to the ops dictionary in "
@@ -114,6 +131,14 @@ T = np.diag([1, np.exp(1j * np.pi / 4)])
 SX = 0.5 * np.array([[1 + 1j, 1 - 1j], [1 - 1j, 1 + 1j]])
 SWAP = np.array([[1, 0, 0, 0], [0, 0, 1, 0], [0, 1, 0, 0], [0, 0, 0, 1]])
 ISWAP = np.array([[1, 0, 0, 0], [0, 0, 1j, 0], [0, 1j, 0, 0], [0, 0, 0, 1]])
+ECR = np.array(
+    [
+        [0, 0, 1 / sqrt(2), 1j * 1 / sqrt(2)],
+        [0, 0, 1j * 1 / sqrt(2), 1 / sqrt(2)],
+        [1 / sqrt(2), -1j * 1 / sqrt(2), 0, 0],
+        [-1j * 1 / sqrt(2), 1 / sqrt(2), 0, 0],
+    ]
+)
 CNOT = np.array([[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 0, 1], [0, 0, 1, 0]])
 CZ = np.diag([1, 1, 1, -1])
 CY = np.array([[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 0, -1j], [0, 0, 1j, 0]])
@@ -179,6 +204,15 @@ IsingXX = lambda phi: np.array(
     ]
 )
 
+IsingXY = lambda phi: np.array(
+    [
+        [1, 0, 0, 0],
+        [0, cos(phi / 2), 1j * sin(phi / 2), 0],
+        [0, 1j * sin(phi / 2), cos(phi / 2), 0],
+        [0, 0, 0, 1],
+    ]
+)
+
 IsingYY = lambda phi: np.array(
     [
         [cos(phi / 2), 0, 0, 1j * sin(phi / 2)],
@@ -197,6 +231,22 @@ IsingZZ = lambda phi: np.array(
     ]
 )
 
+PSWAP = lambda phi: np.array(
+    [
+        [1, 0, 0, 0],
+        [0, 0, exp(1.0j * phi), 0],
+        [0, exp(1.0j * phi), 0, 0],
+        [0, 0, 0, 1],
+    ]
+)
+
+
+def adjoint_tuple(op, orig_mat):
+    """Returns op constructor and matrix for provided base ops."""
+    mat = qml.math.conj(qml.math.transpose(orig_mat))
+    return (qml.adjoint(op), mat)
+
+
 # list of all non-parametrized single-qubit gates,
 # along with the PennyLane operation name
 single_qubit = [
@@ -207,6 +257,9 @@ single_qubit = [
     (qml.S, S),
     (qml.T, T),
     (qml.SX, SX),
+    adjoint_tuple(qml.S, S),
+    adjoint_tuple(qml.T, T),
+    adjoint_tuple(qml.SX, SX),
 ]
 
 # list of all parametrized single-qubit gates
@@ -218,15 +271,25 @@ single_qubit_param = [
     (qml.RZ, rz),
 ]
 # list of all non-parametrized two-qubit gates
-two_qubit = [(qml.CNOT, CNOT), (qml.SWAP, SWAP), (qml.ISWAP, ISWAP), (qml.CZ, CZ), (qml.CY, CY)]
+two_qubit = [
+    (qml.CNOT, CNOT),
+    (qml.SWAP, SWAP),
+    (qml.ISWAP, ISWAP),
+    (qml.ECR, ECR),
+    (qml.CZ, CZ),
+    (qml.CY, CY),
+    adjoint_tuple(qml.ISWAP, ISWAP),
+]
 # list of all parametrized two-qubit gates
 two_qubit_param = [
     (qml.CRX, crx),
     (qml.CRY, cry),
     (qml.CRZ, crz),
     (qml.IsingXX, IsingXX),
+    (qml.IsingXY, IsingXY),
     (qml.IsingYY, IsingYY),
     (qml.IsingZZ, IsingZZ),
+    (qml.PSWAP, PSWAP),
 ]
 two_qubit_multi_param = [(qml.CRot, crot)]
 # list of all three-qubit gates
@@ -272,7 +335,7 @@ class TestSupportedGates:
 
             @qml.qnode(dev)
             def circuit():
-                ops[operation]
+                qml.apply(ops[operation])
                 return qml.expval(qml.Identity(wires=0))
 
             assert isinstance(circuit(), (float, np.ndarray))
@@ -327,7 +390,7 @@ class TestGatesQubit:
 
         res = circuit()
 
-        expected = np.zeros([2 ** n_wires])
+        expected = np.zeros([2**n_wires])
         expected[np.ravel_multi_index(basis_state, [2] * n_wires)] = 1
         assert np.allclose(res, expected, atol=tol(dev.shots))
 
@@ -418,6 +481,8 @@ class TestGatesQubit:
         n_wires = 2
         dev = device(n_wires)
         skip_if(dev, {"returns_probs": False})
+        if not dev.supports_operation(op(wires=range(n_wires)).name):
+            pytest.skip("op not supported")
 
         rnd_state = init_state(n_wires)
 
