@@ -250,9 +250,9 @@ def qnode_spectrum(qnode, encoding_args=None, argnum=None, decimals=8, validatio
     >>> z = -1.8
     >>> w = np.random.random((2, n_qubits, 3))
     >>> print(qml.draw(circuit)(x, y, z, w))
-    0: ──RX(0.5)──Rot(0.598, 0.949, 0.346)───RY(0.23)──Rot(0.693, 0.0738, 0.246)──RX(-1.8)──┤ ⟨Z⟩
-    1: ──RX(1)────Rot(0.0711, 0.701, 0.445)──RY(0.69)──Rot(0.32, 0.0482, 0.437)───RX(-1.8)──┤
-    2: ──RX(1.5)──Rot(0.401, 0.0795, 0.731)──RY(1.15)──Rot(0.756, 0.38, 0.38)─────RX(-1.8)──┤
+    0: ──RX(0.50)──Rot(0.09,0.46,0.54)──RY(0.23)──Rot(0.59,0.22,0.05)──RX(-1.80)─┤  <Z>
+    1: ──RX(1.00)──Rot(0.98,0.61,0.07)──RY(0.69)──Rot(0.62,0.00,0.28)──RX(-1.80)─┤
+    2: ──RX(1.50)──Rot(0.65,0.07,0.36)──RY(1.15)──Rot(0.74,0.27,0.24)──RX(-1.80)─┤
 
     Applying the ``qnode_spectrum`` function to the circuit for
     the non-trainable parameters, we obtain:
@@ -270,7 +270,8 @@ def qnode_spectrum(qnode, encoding_args=None, argnum=None, decimals=8, validatio
         it may still change based on inputs to the QNode that alter the architecture
         of the circuit.
 
-    .. UsageDetails::
+    .. details::
+        :title: Usage Details
 
         Above, we selected all input-encoding parameters for the spectrum computation, using
         the ``argnum`` keyword argument. We may also restrict the full analysis to a single
@@ -394,35 +395,35 @@ def qnode_spectrum(qnode, encoding_args=None, argnum=None, decimals=8, validatio
         # After construction, check whether invalid operations (for a spectrum)
         # are present in the QNode
         for m in qnode.qtape.measurements:
-            if m.return_type not in {qml.operation.Expectation, qml.operation.Probability}:
+            if m.return_type not in {qml.measurements.Expectation, qml.measurements.Probability}:
                 raise ValueError(
                     f"The return_type {m.return_type.value} is not supported as it likely does "
                     "not admit a Fourier spectrum."
                 )
-        class_jacs = jac_fn(*args, **kwargs)
+        cjacs = jac_fn(*args, **kwargs)
         spectra = {}
         tape = qml.transforms.expand_multipar(qnode.qtape)
         par_info = tape._par_info
 
         # Iterate over jacobians per argument
-        for jac_idx, class_jac in enumerate(class_jacs):
+        for jac_idx, cjac in enumerate(cjacs):
             # Obtain argument name for the jacobian index
             arg_name = arg_name_map[jac_idx]
             # Extract requested parameter indices for the current argument
             if encoding_args[arg_name] is Ellipsis:
                 # If no index for this argument is specified, request all parameters within
                 # the argument (Recall () is a valid index for scalar-valued arguments here)
-                requested_par_ids = set(product(*(range(sh) for sh in class_jac.shape[1:])))
+                requested_par_ids = set(product(*(range(sh) for sh in cjac.shape[1:])))
             else:
                 requested_par_ids = set(encoding_args[arg_name])
             # Each requested parameter at least "contributes" as a constant
             _spectra = {par_idx: {0} for par_idx in requested_par_ids}
 
             # Iterate over the axis of the current Jacobian that corresponds to the tape operations
-            for op_idx, jac_of_op in enumerate(np.round(class_jac, decimals=decimals)):
+            for op_idx, jac_of_op in enumerate(np.round(cjac, decimals=decimals)):
                 op = par_info[op_idx]["op"]
                 # Find parameters that both were requested and feed into the operation
-                if len(class_jac.shape) == 1:
+                if len(cjac.shape) == 1:
                     # Scalar argument, only axis of Jacobian is for operations
                     if np.isclose(jac_of_op, 0.0, atol=atol, rtol=0):
                         continue
