@@ -15,17 +15,22 @@ r"""
 This subpackage defines functions and classes for Pauli-word partitioning
 functionality used in measurement optimization.
 """
+from warnings import warn
 
-from . import graph_colouring
-from .group_observables import group_observables, PauliGroupingStrategy
-from .optimize_measurements import optimize_measurements
-from .transformations import (
+
+from . import graph_colouring  # pylint:disable=wrong-import-position
+from .group_observables import (  # pylint:disable=wrong-import-position
+    group_observables,
+    PauliGroupingStrategy,
+)
+from .optimize_measurements import optimize_measurements  # pylint:disable=wrong-import-position
+from .transformations import (  # pylint:disable=wrong-import-position
     qwc_rotation,
     diagonalize_pauli_word,
     diagonalize_qwc_pauli_words,
     diagonalize_qwc_groupings,
 )
-from .utils import (
+from .utils import (  # pylint:disable=wrong-import-position
     is_pauli_word,
     are_identical_pauli_words,
     pauli_to_binary,
@@ -33,9 +38,14 @@ from .utils import (
     pauli_word_to_string,
     string_to_pauli_word,
     pauli_word_to_matrix,
-    is_commuting,
     is_qwc,
+    are_pauli_words_qwc,
     observables_to_binary_matrix,
     qwc_complement_adj_matrix,
 )
-from .pauli import pauli_group, pauli_mult, pauli_mult_with_phase
+from .pauli import (  # pylint:disable=wrong-import-position
+    pauli_group,
+    pauli_mult,
+    pauli_mult_with_phase,
+    partition_pauli_group,
+)
