@@ -15,16 +15,16 @@
 This submodule contains the discrete-variable quantum operations that perform
 arithmetic operations on their input states.
 """
-# pylint: disable=too-many-arguments
+# pylint: disable=too-many-arguments,too-many-instance-attributes
 import itertools
-from copy import copy
+import numbers
 from collections.abc import Iterable
+from copy import copy
+from typing import List
 
 import pennylane as qml
 from pennylane import numpy as np
-
 from pennylane.operation import Observable, Tensor
-from pennylane.queuing import QueuingError
 from pennylane.wires import Wires
 
 OBS_MAP = {"PauliX": "X", "PauliY": "Y", "PauliZ": "Z", "Hadamard": "H", "Identity": "I"}
@@ -34,7 +34,7 @@ def _compute_grouping_indices(observables, grouping_type="qwc", method="rlf"):
 
     # todo: directly compute the
     # indices, instead of extracting groups of observables first
-    observable_groups = qml.grouping.group_observables(
+    observable_groups = qml.pauli.group_observables(
         observables, coefficients=None, grouping_type=grouping_type, method=method
     )
 
@@ -47,7 +47,7 @@ def _compute_grouping_indices(observables, grouping_type="qwc", method="rlf"):
         for pauli_word in partition:
             # find index of this pauli word in remaining original observables,
             for observable in observables:
-                if qml.grouping.utils.are_identical_pauli_words(pauli_word, observable):
+                if qml.pauli.are_identical_pauli_words(pauli_word, observable):
                     ind = observables.index(observable)
                     indices_this_group.append(available_indices[ind])
                     # delete this observable and its index, so it cannot be found again
@@ -160,14 +160,12 @@ class Hamiltonian(Observable):
     """
 
     num_wires = qml.operation.AnyWires
-    num_params = 1
-    par_domain = "A"
     grad_method = "A"  # supports analytic gradients
 
     def __init__(
         self,
         coeffs,
-        observables,
+        observables: List[Observable],
         simplify=False,
         grouping_type=None,
         method="rlf",
@@ -189,6 +187,11 @@ class Hamiltonian(Observable):
 
         self._coeffs = coeffs
         self._ops = list(observables)
+
+        # TODO: avoid having multiple ways to store ops and coeffs,
+        # ideally only use parameters for coeffs, and hyperparameters for ops
+        self._hyperparameters = {"ops": self._ops}
+
         self._wires = qml.wires.Wires.all_wires([op.wires for op in self.ops], sort=True)
 
         self.return_type = None
@@ -200,22 +203,21 @@ class Hamiltonian(Observable):
         if simplify:
             self.simplify()
         if grouping_type is not None:
-            with qml.tape.stop_recording():
+            with qml.QueuingManager.stop_recording():
                 self._grouping_indices = _compute_grouping_indices(
                     self.ops, grouping_type=grouping_type, method=method
                 )
 
         coeffs_flat = [self._coeffs[i] for i in range(qml.math.shape(self._coeffs)[0])]
-        # overwrite this attribute, now that we have the correct info
-        self.num_params = qml.math.shape(self._coeffs)[0]
 
         # create the operator using each coefficient as a separate parameter;
         # this causes H.data to be a list of tensor scalars,
         # while H.coeffs is the original tensor
         super().__init__(*coeffs_flat, wires=self._wires, id=id, do_queue=do_queue)
 
-    def label(self, decimals=None, base_label=None):
-        return super().label(decimals=decimals, base_label=base_label or "𝓗")
+    def label(self, decimals=None, base_label=None, cache=None):
+        decimals = None if (len(self.parameters) > 3) else decimals
+        return super().label(decimals=decimals, base_label=base_label or "𝓗", cache=cache)
 
     @property
     def coeffs(self):
@@ -235,12 +237,31 @@ class Hamiltonian(Observable):
         """
         return self._ops
 
-    @property
     def terms(self):
-        r"""The terms of the Hamiltonian expression :math:`\sum_{k=0}^{N-1} c_k O_k`
+        r"""Representation of the operator as a linear combination of other operators.
+
+         .. math:: O = \sum_i c_i O_i
+
+         .. seealso:: :meth:`~.Hamiltonian.terms`
 
         Returns:
-            (tuple, tuple): tuples of coefficients and operations, each of length N
+            tuple[Iterable[tensor_like or float], list[.Operator]]: coefficients and operations
+
+        **Example**
+        >>> coeffs = [1., 2.]
+        >>> ops = [qml.PauliX(0), qml.PauliZ(0)]
+        >>> H = qml.Hamiltonian(coeffs, ops)
+
+        >>> H.terms()
+        [1., 2.], [qml.PauliX(0), qml.PauliZ(0)]
+
+        The coefficients are differentiable and can be stored as tensors:
+        >>> import tensorflow as tf
+        >>> H = qml.Hamiltonian([tf.Variable(1.), tf.Variable(2.)], [qml.PauliX(0), qml.PauliZ(0)])
+        >>> t = H.terms()
+
+        >>> t[0]
+        [<tf.Tensor: shape=(), dtype=float32, numpy=1.0>, <tf.Tensor: shape=(), dtype=float32, numpy=2.0>]
         """
         return self.coeffs, self.ops
 
@@ -299,8 +320,8 @@ class Hamiltonian(Observable):
             or any(i not in range(len(self.ops)) for i in [i for sl in value for i in sl])
         ):
             raise ValueError(
-                "The grouped index value needs to be a list of lists of integers between 0 and the "
-                "number of observables in the Hamiltonian; got {}".format(value)
+                f"The grouped index value needs to be a list of lists of integers between 0 and the "
+                f"number of observables in the Hamiltonian; got {value}"
             )
         self._grouping_indices = value
 
@@ -316,7 +337,7 @@ class Hamiltonian(Observable):
                 can be ``'lf'`` (Largest First) or ``'rlf'`` (Recursive Largest First).
         """
 
-        with qml.tape.stop_recording():
+        with qml.QueuingManager.stop_recording():
             self._grouping_indices = _compute_grouping_indices(
                 self.ops, grouping_type=grouping_type, method=method
             )
@@ -338,42 +359,49 @@ class Hamiltonian(Observable):
             Calling this method will reset ``grouping_indices`` to None, since
             the observables it refers to are updated.
         """
-        data = []
-        ops = []
+
+        # Todo: make simplify return a new operation, so
+        # it does not mutate this one
+
+        new_coeffs = []
+        new_ops = []
 
         for i in range(len(self.ops)):  # pylint: disable=consider-using-enumerate
             op = self.ops[i]
             c = self.coeffs[i]
             op = op if isinstance(op, Tensor) else Tensor(op)
 
-            ind = None
-            for j, o in enumerate(ops):
-                if op.compare(o):
-                    ind = j
-                    break
-
+            ind = next((j for j, o in enumerate(new_ops) if op.compare(o)), None)
             if ind is not None:
-                data[ind] += c
-                if np.isclose(qml.math.toarray(data[ind]), np.array(0.0)):
-                    del data[ind]
-                    del ops[ind]
+                new_coeffs[ind] += c
+                if np.isclose(qml.math.toarray(new_coeffs[ind]), np.array(0.0)):
+                    del new_coeffs[ind]
+                    del new_ops[ind]
             else:
-                ops.append(op.prune())
-                data.append(c)
+                new_ops.append(op.prune())
+                new_coeffs.append(c)
 
-        self._coeffs = qml.math.stack(data) if data else []
-        self.data = data
-        self._ops = ops
+        # hotfix: We `self.data`, since `self.parameters` returns a copy of the data and is now returned in
+        # self.terms(). To be improved soon.
+        self.data = new_coeffs
+        # hotfix: We overwrite the hyperparameter entry, which is now returned in self.terms().
+        # To be improved soon.
+        self.hyperparameters["ops"] = new_ops
+
+        self._coeffs = qml.math.stack(new_coeffs) if new_coeffs else []
+        self._ops = new_ops
+        self._wires = qml.wires.Wires.all_wires([op.wires for op in self.ops], sort=True)
         # reset grouping, since the indices refer to the old observables and coefficients
         self._grouping_indices = None
 
     def __str__(self):
-        # Lambda function that formats the wires
-        wires_print = lambda ob: ",".join(map(str, ob.wires.tolist()))
+        def wires_print(ob: Observable):
+            """Function that formats the wires."""
+            return ",".join(map(str, ob.wires.tolist()))
 
         list_of_coeffs = self.data  # list of scalar tensors
         paired_coeff_obs = list(zip(list_of_coeffs, self.ops))
-        paired_coeff_obs.sort(key=lambda pair: (len(pair[1].wires), pair[0]))
+        paired_coeff_obs.sort(key=lambda pair: (len(pair[1].wires), qml.math.real(pair[0])))
 
         terms_ls = []
 
@@ -395,6 +423,15 @@ class Hamiltonian(Observable):
         # Constructor-call-like representation
         return f"<Hamiltonian: terms={qml.math.shape(self.coeffs)[0]}, wires={self.wires.tolist()}>"
 
+    def _ipython_display_(self):  # pragma: no-cover
+        """Displays __str__ in ipython instead of __repr__
+        See https://ipython.readthedocs.io/en/stable/config/integrating.html
+        """
+        if len(self.ops) < 15:
+            print(str(self))
+        else:  # pragma: no-cover
+            print(repr(self))
+
     def _obs_data(self):
         r"""Extracts the data from a Hamiltonian and serializes it in an order-independent fashion.
 
@@ -431,9 +468,9 @@ class Hamiltonian(Observable):
         return data
 
     def compare(self, other):
-        r"""Compares with another :class:`~Hamiltonian`, :class:`~.Observable`, or :class:`~.Tensor`,
-        to determine if they are equivalent.
+        r"""Determines whether the operator is equivalent to another.
 
+        Currently only supported for :class:`~Hamiltonian`, :class:`~.Observable`, or :class:`~.Tensor`.
         Hamiltonians/observables are equivalent if they represent the same operator
         (their matrix representations are equal), and they are defined on the same wires.
 
@@ -451,12 +488,11 @@ class Hamiltonian(Observable):
 
         **Examples**
 
-        >>> A = np.array([[1, 0], [0, -1]])
         >>> H = qml.Hamiltonian(
         ...     [0.5, 0.5],
-        ...     [qml.Hermitian(A, 0) @ qml.PauliY(1), qml.PauliY(1) @ qml.Hermitian(A, 0) @ qml.Identity("a")]
+        ...     [qml.PauliZ(0) @ qml.PauliY(1), qml.PauliY(1) @ qml.PauliZ(0) @ qml.Identity("a")]
         ... )
-        >>> obs = qml.Hermitian(A, 0) @ qml.PauliY(1)
+        >>> obs = qml.PauliZ(0) @ qml.PauliY(1)
         >>> print(H.compare(obs))
         True
 
@@ -502,7 +538,6 @@ class Hamiltonian(Observable):
             coeffs = qml.math.kron(coeffs1, coeffs2)
             ops_list = itertools.product(ops1, ops2)
             terms = [qml.operation.Tensor(t[0], t[1]) for t in ops_list]
-
             return qml.Hamiltonian(coeffs, terms, simplify=True)
 
         if isinstance(H, (Tensor, Observable)):
@@ -512,11 +547,31 @@ class Hamiltonian(Observable):
 
         raise ValueError(f"Cannot tensor product Hamiltonian and {type(H)}")
 
+    def __rmatmul__(self, H):
+        r"""The tensor product operation (from the right) between a Hamiltonian and
+        a Hamiltonian/Tensor/Observable (ie. Hamiltonian.__rmul__(H) = H @ Hamiltonian).
+        """
+        if isinstance(H, Hamiltonian):  # can't be accessed by '@'
+            return H.__matmul__(self)
+
+        coeffs1 = copy(self.coeffs)
+        ops1 = self.ops.copy()
+
+        if isinstance(H, (Tensor, Observable)):
+            terms = [H @ op for op in ops1]
+
+            return qml.Hamiltonian(coeffs1, terms, simplify=True)
+
+        raise ValueError(f"Cannot tensor product Hamiltonian and {type(H)}")
+
     def __add__(self, H):
         r"""The addition operation between a Hamiltonian and a Hamiltonian/Tensor/Observable."""
         ops = self.ops.copy()
         self_coeffs = copy(self.coeffs)
 
+        if isinstance(H, numbers.Number) and H == 0:
+            return self
+
         if isinstance(H, Hamiltonian):
             coeffs = qml.math.concatenate([self_coeffs, copy(H.coeffs)], axis=0)
             ops.extend(H.ops.copy())
@@ -531,11 +586,13 @@ class Hamiltonian(Observable):
 
         raise ValueError(f"Cannot add Hamiltonian and {type(H)}")
 
+    __radd__ = __add__
+
     def __mul__(self, a):
         r"""The scalar multiplication operation between a scalar and a Hamiltonian."""
         if isinstance(a, (int, float)):
             self_coeffs = copy(self.coeffs)
-            coeffs = qml.math.multiply(qml.math.cast_like([a], self_coeffs), self_coeffs)
+            coeffs = qml.math.multiply(a, self_coeffs)
             return qml.Hamiltonian(coeffs, self.ops.copy())
 
         raise ValueError(f"Cannot multiply Hamiltonian by {type(a)}")
@@ -550,6 +607,9 @@ class Hamiltonian(Observable):
 
     def __iadd__(self, H):
         r"""The inplace addition operation between a Hamiltonian and a Hamiltonian/Tensor/Observable."""
+        if isinstance(H, numbers.Number) and H == 0:
+            return self
+
         if isinstance(H, Hamiltonian):
             self._coeffs = qml.math.concatenate([self._coeffs, H.coeffs], axis=0)
             self._ops.extend(H.ops.copy())
@@ -569,7 +629,7 @@ class Hamiltonian(Observable):
     def __imul__(self, a):
         r"""The inplace scalar multiplication operation between a scalar and a Hamiltonian."""
         if isinstance(a, (int, float)):
-            self._coeffs = qml.math.multiply(qml.math.cast_like([a], self._coeffs), self._coeffs)
+            self._coeffs = qml.math.multiply(a, self._coeffs)
             return self
 
         raise ValueError(f"Cannot multiply Hamiltonian by {type(a)}")
@@ -581,16 +641,34 @@ class Hamiltonian(Observable):
             return self
         raise ValueError(f"Cannot subtract {type(H)} from Hamiltonian")
 
-    def queue(self, context=qml.QueuingContext):
+    def queue(self, context=qml.QueuingManager):
         """Queues a qml.Hamiltonian instance"""
         for o in self.ops:
-            try:
-                context.update_info(o, owner=self)
-            except QueuingError:
-                o.queue(context=context)
-                context.update_info(o, owner=self)
-            except NotImplementedError:
-                pass
-
+            context.update_info(o, owner=self)
         context.append(self, owns=tuple(self.ops))
         return self
+
+    def map_wires(self, wire_map: dict):
+        """Returns a copy of the current hamiltonian with its wires changed according to the given
+        wire map.
+
+        Args:
+            wire_map (dict): dictionary containing the old wires as keys and the new wires as values
+
+        Returns:
+            .Hamiltonian: new hamiltonian
+        """
+        cls = self.__class__
+        new_op = cls.__new__(cls)
+        new_op.data = self.data.copy()
+        new_op._wires = Wires(  # pylint: disable=protected-access
+            [wire_map.get(wire, wire) for wire in self.wires]
+        )
+        new_op._ops = [  # pylint: disable=protected-access
+            op.map_wires(wire_map) for op in self.ops
+        ]
+        for attr, value in vars(self).items():
+            if attr not in {"data", "_wires", "_ops"}:
+                setattr(new_op, attr, value)
+        new_op.hyperparameters["ops"] = new_op._ops  # pylint: disable=protected-access
+        return new_op
