@@ -114,7 +114,8 @@ class ShotAdaptiveOptimizer(GradientDescentOptimizer):
     Step 21: cost = -7.23, shots_used = 82014
     Step 22: cost = -7.31, shots_used = 92838
 
-    .. UsageDetails::
+    .. details::
+        :title: Usage Details
 
         The shot adaptive optimizer is based on the iCANS1 optimizer by
         `Kübler et al. (2020) <https://quantum-journal.org/papers/q-2020-05-11-263/>`__, and works
@@ -350,7 +351,7 @@ class ShotAdaptiveOptimizer(GradientDescentOptimizer):
 
     def compute_grad(
         self, objective_fn, args, kwargs
-    ):  # pylint: disable=signature-differs,arguments-differ
+    ):  # pylint: disable=signature-differs,arguments-differ,arguments-renamed
         r"""Compute gradient of the objective function, as well as the variance of the gradient,
         at the given point.
 
@@ -411,7 +412,7 @@ class ShotAdaptiveOptimizer(GradientDescentOptimizer):
         self.trainable_args = set()
 
         for index, arg in enumerate(args):
-            if getattr(arg, "requires_grad", True):
+            if getattr(arg, "requires_grad", False):
                 self.trainable_args |= {index}
 
         if self.s is None:
@@ -451,15 +452,15 @@ class ShotAdaptiveOptimizer(GradientDescentOptimizer):
             # iteration of the optimizer
             s = np.ceil(
                 (2 * self.lipschitz * self.stepsize * xi)
-                / ((2 - self.lipschitz * self.stepsize) * (chi ** 2 + self.b * (self.mu ** self.k)))
+                / ((2 - self.lipschitz * self.stepsize) * (chi**2 + self.b * (self.mu**self.k)))
             )
 
             # apply an upper and lower bound on the new shot distributions,
             # to avoid the number of shots reducing below min(2, min_shots),
             # or growing too significantly.
             gamma = (
-                (self.stepsize - self.lipschitz * self.stepsize ** 2 / 2) * chi ** 2
-                - xi * self.lipschitz * self.stepsize ** 2 / (2 * s)
+                (self.stepsize - self.lipschitz * self.stepsize**2 / 2) * chi**2
+                - xi * self.lipschitz * self.stepsize**2 / (2 * s)
             ) / s
 
             argmax_gamma = np.unravel_index(np.argmax(gamma), gamma.shape)
