@@ -16,9 +16,9 @@ This module contains templates, which are pre-coded routines that can be used in
 """
 
 from .broadcast import *
-from .decorator import *
 from .layer import *
 from .layers import *
 from .embeddings import *
 from .subroutines import *
 from .state_preparations import *
+from .tensornetworks import *
