@@ -17,6 +17,7 @@ any of the characteristics of other templates (i.e. to prepare a specific state,
 """
 
 from .arbitrary_unitary import ArbitraryUnitary
+from .commuting_evolution import CommutingEvolution
 from .fermionic_double_excitation import FermionicDoubleExcitation
 from .interferometer import Interferometer
 from .fermionic_single_excitation import FermionicSingleExcitation
@@ -29,3 +30,5 @@ from .all_singles_doubles import AllSinglesDoubles
 from .grover import GroverOperator
 from .qft import QFT
 from .kupccgsd import kUpCCGSD
+from .hilbert_schmidt import HilbertSchmidt, LocalHilbertSchmidt
+from .flip_sign import FlipSign
