@@ -15,23 +15,12 @@ r"""
 Contains the FermionicDoubleExcitation template.
 """
 # pylint: disable-msg=too-many-branches,too-many-arguments,protected-access
-import math
-
 import numpy as np
 import pennylane as qml
 from pennylane.operation import Operation, AnyWires
 from pennylane.ops import RZ, RX, CNOT, Hadamard
 
 
-# Four term gradient recipe for controlled rotations
-INV_SQRT2 = 1 / math.sqrt(2)
-c1 = INV_SQRT2 * (np.sqrt(2) + 1) / 4
-c2 = INV_SQRT2 * (np.sqrt(2) - 1) / 4
-a = np.pi / 2
-b = 3 * np.pi / 2
-four_term_grad_recipe = ([[c1, 1, a], [-c1, 1, -a], [-c2, 1, b], [c2, 1, -b]],)
-
-
 def _layer1(weight, s, r, q, p, set_cnot_wires):
     r"""Implement the first layer of the circuit to exponentiate the double-excitation
     operator entering the UCCSD ansatz.
@@ -49,30 +38,30 @@ def _layer1(weight, s, r, q, p, set_cnot_wires):
         q (int): qubit index ``q``
         p (int): qubit index ``p``
         set_cnot_wires (list[Wires]): list of CNOT wires
-    """
 
+    Returns:
+          list[.Operator]: sequence of operators defined by this function
+    """
     # U_1, U_2, U_3, U_4 acting on wires 's', 'r', 'q' and 'p'
-    Hadamard(wires=s)
-    Hadamard(wires=r)
-    RX(-np.pi / 2, wires=q)
-    Hadamard(wires=p)
+    op_list = [Hadamard(wires=s), Hadamard(wires=r), RX(-np.pi / 2, wires=q), Hadamard(wires=p)]
 
     # Applying CNOTs
     for cnot_wires in set_cnot_wires:
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # Z rotation acting on wire 'p'
-    RZ(weight / 8, wires=p)
+    op_list.append(RZ(weight / 8, wires=p))
 
     # Applying CNOTs in reverse order
     for cnot_wires in reversed(set_cnot_wires):
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # U_1^+, U_2^+, U_3^+, U_4^+ acting on wires 's', 'r', 'q' and 'p'
-    Hadamard(wires=s)
-    Hadamard(wires=r)
-    RX(np.pi / 2, wires=q)
-    Hadamard(wires=p)
+    op_list.append(Hadamard(wires=s))
+    op_list.append(Hadamard(wires=r))
+    op_list.append(RX(np.pi / 2, wires=q))
+    op_list.append(Hadamard(wires=p))
+    return op_list
 
 
 def _layer2(weight, s, r, q, p, set_cnot_wires):
@@ -92,30 +81,35 @@ def _layer2(weight, s, r, q, p, set_cnot_wires):
         q (int): qubit index ``q``
         p (int): qubit index ``p``
         set_cnot_wires (list[Wires]): list of CNOT wires
-    """
 
+    Returns:
+        list[.Operator]: sequence of operators defined by this function
+    """
     # U_1, U_2, U_3, U_4 acting on wires 's', 'r', 'q' and 'p'
-    RX(-np.pi / 2, wires=s)
-    Hadamard(wires=r)
-    RX(-np.pi / 2, wires=q)
-    RX(-np.pi / 2, wires=p)
+    op_list = [
+        RX(-np.pi / 2, wires=s),
+        Hadamard(wires=r),
+        RX(-np.pi / 2, wires=q),
+        RX(-np.pi / 2, wires=p),
+    ]
 
     # Applying CNOTs
     for cnot_wires in set_cnot_wires:
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # Z rotation acting on wire 'p'
-    RZ(weight / 8, wires=p)
+    op_list.append(RZ(weight / 8, wires=p))
 
     # Applying CNOTs in reverse order
     for cnot_wires in reversed(set_cnot_wires):
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # U_1^+, U_2^+, U_3^+, U_4^+ acting on wires 's', 'r', 'q' and 'p'
-    RX(np.pi / 2, wires=s)
-    Hadamard(wires=r)
-    RX(np.pi / 2, wires=q)
-    RX(np.pi / 2, wires=p)
+    op_list.append(RX(np.pi / 2, wires=s))
+    op_list.append(Hadamard(wires=r))
+    op_list.append(RX(np.pi / 2, wires=q))
+    op_list.append(RX(np.pi / 2, wires=p))
+    return op_list
 
 
 def _layer3(weight, s, r, q, p, set_cnot_wires):
@@ -135,30 +129,35 @@ def _layer3(weight, s, r, q, p, set_cnot_wires):
         q (int): qubit index ``q``
         p (int): qubit index ``p``
         set_cnot_wires (list[Wires]): list of CNOT wires
-    """
 
+    Returns:
+        list[.Operator]: sequence of operators defined by this function
+    """
     # U_1, U_2, U_3, U_4 acting on wires 's', 'r', 'q' and 'p'
-    Hadamard(wires=s)
-    RX(-np.pi / 2, wires=r)
-    RX(-np.pi / 2, wires=q)
-    RX(-np.pi / 2, wires=p)
+    op_list = [
+        Hadamard(wires=s),
+        RX(-np.pi / 2, wires=r),
+        RX(-np.pi / 2, wires=q),
+        RX(-np.pi / 2, wires=p),
+    ]
 
     # Applying CNOTs
     for cnot_wires in set_cnot_wires:
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # Z rotation acting on wire 'p'
-    RZ(weight / 8, wires=p)
+    op_list.append(RZ(weight / 8, wires=p))
 
     # Applying CNOTs in reverse order
     for cnot_wires in reversed(set_cnot_wires):
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # U_1^+, U_2^+, U_3^+, U_4^+ acting on wires 's', 'r', 'q' and 'p'
-    Hadamard(wires=s)
-    RX(np.pi / 2, wires=r)
-    RX(np.pi / 2, wires=q)
-    RX(np.pi / 2, wires=p)
+    op_list.append(Hadamard(wires=s))
+    op_list.append(RX(np.pi / 2, wires=r))
+    op_list.append(RX(np.pi / 2, wires=q))
+    op_list.append(RX(np.pi / 2, wires=p))
+    return op_list
 
 
 def _layer4(weight, s, r, q, p, set_cnot_wires):
@@ -178,30 +177,30 @@ def _layer4(weight, s, r, q, p, set_cnot_wires):
         q (int): qubit index ``q``
         p (int): qubit index ``p``
         set_cnot_wires (list[Wires]): list of CNOT wires
-    """
 
+    Returns:
+        list[.Operator]: sequence of operators defined by this function
+    """
     # U_1, U_2, U_3, U_4 acting on wires 's', 'r', 'q' and 'p'
-    Hadamard(wires=s)
-    Hadamard(wires=r)
-    Hadamard(wires=q)
-    RX(-np.pi / 2, wires=p)
+    op_list = [Hadamard(wires=s), Hadamard(wires=r), Hadamard(wires=q), RX(-np.pi / 2, wires=p)]
 
     # Applying CNOTs
     for cnot_wires in set_cnot_wires:
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # Z rotation acting on wire 'p'
-    RZ(weight / 8, wires=p)
+    op_list.append(RZ(weight / 8, wires=p))
 
     # Applying CNOTs in reverse order
     for cnot_wires in reversed(set_cnot_wires):
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # U_1^+, U_2^+, U_3^+, U_4^+ acting on wires 's', 'r', 'q' and 'p'
-    Hadamard(wires=s)
-    Hadamard(wires=r)
-    Hadamard(wires=q)
-    RX(np.pi / 2, wires=p)
+    op_list.append(Hadamard(wires=s))
+    op_list.append(Hadamard(wires=r))
+    op_list.append(Hadamard(wires=q))
+    op_list.append(RX(np.pi / 2, wires=p))
+    return op_list
 
 
 def _layer5(weight, s, r, q, p, set_cnot_wires):
@@ -221,30 +220,30 @@ def _layer5(weight, s, r, q, p, set_cnot_wires):
         q (int): qubit index ``q``
         p (int): qubit index ``p``
         set_cnot_wires (list[Wires]): list of CNOT wires
-    """
 
+    Returns:
+        list[.Operator]: sequence of operators defined by this function
+    """
     # U_1, U_2, U_3, U_4 acting on wires 's', 'r', 'q' and 'p'
-    RX(-np.pi / 2, wires=s)
-    Hadamard(wires=r)
-    Hadamard(wires=q)
-    Hadamard(wires=p)
+    op_list = [RX(-np.pi / 2, wires=s), Hadamard(wires=r), Hadamard(wires=q), Hadamard(wires=p)]
 
     # Applying CNOTs
     for cnot_wires in set_cnot_wires:
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # Z rotation acting on wire 'p'
-    RZ(-weight / 8, wires=p)
+    op_list.append(RZ(-weight / 8, wires=p))
 
     # Applying CNOTs in reverse order
     for cnot_wires in reversed(set_cnot_wires):
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # U_1^+, U_2^+, U_3^+, U_4^+ acting on wires 's', 'r', 'q' and 'p'
-    RX(np.pi / 2, wires=s)
-    Hadamard(wires=r)
-    Hadamard(wires=q)
-    Hadamard(wires=p)
+    op_list.append(RX(np.pi / 2, wires=s))
+    op_list.append(Hadamard(wires=r))
+    op_list.append(Hadamard(wires=q))
+    op_list.append(Hadamard(wires=p))
+    return op_list
 
 
 def _layer6(weight, s, r, q, p, set_cnot_wires):
@@ -264,30 +263,30 @@ def _layer6(weight, s, r, q, p, set_cnot_wires):
         q (int): qubit index ``q``
         p (int): qubit index ``p``
         set_cnot_wires (list[Wires]): list of CNOT wires
-    """
 
+    Returns:
+        list[.Operator]: sequence of operators defined by this function
+    """
     # U_1, U_2, U_3, U_4 acting on wires 's', 'r', 'q' and 'p'
-    Hadamard(wires=s)
-    RX(-np.pi / 2, wires=r)
-    Hadamard(wires=q)
-    Hadamard(wires=p)
+    op_list = [Hadamard(wires=s), RX(-np.pi / 2, wires=r), Hadamard(wires=q), Hadamard(wires=p)]
 
     # Applying CNOTs
     for cnot_wires in set_cnot_wires:
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # Z rotation acting on wire 'p'
-    RZ(-weight / 8, wires=p)
+    op_list.append(RZ(-weight / 8, wires=p))
 
     # Applying CNOTs in reverse order
     for cnot_wires in reversed(set_cnot_wires):
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # U_1^+, U_2^+, U_3^+, U_4^+ acting on wires 's', 'r', 'q' and 'p'
-    Hadamard(wires=s)
-    RX(np.pi / 2, wires=r)
-    Hadamard(wires=q)
-    Hadamard(wires=p)
+    op_list.append(Hadamard(wires=s))
+    op_list.append(RX(np.pi / 2, wires=r))
+    op_list.append(Hadamard(wires=q))
+    op_list.append(Hadamard(wires=p))
+    return op_list
 
 
 def _layer7(weight, s, r, q, p, set_cnot_wires):
@@ -307,30 +306,35 @@ def _layer7(weight, s, r, q, p, set_cnot_wires):
         q (int): qubit index ``q``
         p (int): qubit index ``p``
         set_cnot_wires (list[Wires]): list of CNOT wires
-    """
 
+    Returns:
+        list[.Operator]: sequence of operators defined by this function
+    """
     # U_1, U_2, U_3, U_4 acting on wires 's', 'r', 'q' and 'p'
-    RX(-np.pi / 2, wires=s)
-    RX(-np.pi / 2, wires=r)
-    RX(-np.pi / 2, wires=q)
-    Hadamard(wires=p)
+    op_list = [
+        RX(-np.pi / 2, wires=s),
+        RX(-np.pi / 2, wires=r),
+        RX(-np.pi / 2, wires=q),
+        Hadamard(wires=p),
+    ]
 
     # Applying CNOTs
     for cnot_wires in set_cnot_wires:
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # Z rotation acting on wire 'p'
-    RZ(-weight / 8, wires=p)
+    op_list.append(RZ(-weight / 8, wires=p))
 
     # Applying CNOTs in reverse order
     for cnot_wires in reversed(set_cnot_wires):
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # U_1^+, U_2^+, U_3^+, U_4^+ acting on wires 's', 'r', 'q' and 'p'
-    RX(np.pi / 2, wires=s)
-    RX(np.pi / 2, wires=r)
-    RX(np.pi / 2, wires=q)
-    Hadamard(wires=p)
+    op_list.append(RX(np.pi / 2, wires=s))
+    op_list.append(RX(np.pi / 2, wires=r))
+    op_list.append(RX(np.pi / 2, wires=q))
+    op_list.append(Hadamard(wires=p))
+    return op_list
 
 
 def _layer8(weight, s, r, q, p, set_cnot_wires):
@@ -350,30 +354,35 @@ def _layer8(weight, s, r, q, p, set_cnot_wires):
         q (int): qubit index ``q``
         p (int): qubit index ``p``
         set_cnot_wires (list[Wires]): list of CNOT wires
-    """
 
+    Returns:
+        list[.Operator]: sequence of operators defined by this function
+    """
     # U_1, U_2, U_3, U_4 acting on wires 's', 'r', 'q' and 'p'
-    RX(-np.pi / 2, wires=s)
-    RX(-np.pi / 2, wires=r)
-    Hadamard(wires=q)
-    RX(-np.pi / 2, wires=p)
+    op_list = [
+        RX(-np.pi / 2, wires=s),
+        RX(-np.pi / 2, wires=r),
+        Hadamard(wires=q),
+        RX(-np.pi / 2, wires=p),
+    ]
 
     # Applying CNOTs
     for cnot_wires in set_cnot_wires:
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # Z rotation acting on wire 'p'
-    RZ(-weight / 8, wires=p)
+    op_list.append(RZ(-weight / 8, wires=p))
 
     # Applying CNOTs in reverse order
     for cnot_wires in reversed(set_cnot_wires):
-        CNOT(wires=cnot_wires)
+        op_list.append(CNOT(wires=cnot_wires))
 
     # U_1^+, U_2^+, U_3^+, U_4^+ acting on wires 's', 'r', 'q' and 'p'
-    RX(np.pi / 2, wires=s)
-    RX(np.pi / 2, wires=r)
-    Hadamard(wires=q)
-    RX(np.pi / 2, wires=p)
+    op_list.append(RX(np.pi / 2, wires=s))
+    op_list.append(RX(np.pi / 2, wires=r))
+    op_list.append(Hadamard(wires=q))
+    op_list.append(RX(np.pi / 2, wires=p))
+    return op_list
 
 
 class FermionicDoubleExcitation(Operation):
@@ -436,7 +445,8 @@ class FermionicDoubleExcitation(Operation):
             the last wire is interpreted as ``p``. Wires in between are acted on with CNOT gates
             to compute the parity of the set of qubits.
 
-    .. UsageDetails::
+    .. details::
+        :title: Usage Details
 
         Notice that:
 
@@ -483,58 +493,85 @@ class FermionicDoubleExcitation(Operation):
 
     """
 
-    num_params = 1
     num_wires = AnyWires
-    par_domain = "R"
     grad_method = "A"
-    grad_recipe = four_term_grad_recipe
+    parameter_frequencies = [(0.5, 1.0)]
 
     def __init__(self, weight, wires1=None, wires2=None, do_queue=True, id=None):
 
         if len(wires1) < 2:
             raise ValueError(
-                "expected at least two wires representing the occupied orbitals; "
-                "got {}".format(len(wires1))
+                f"expected at least two wires representing the occupied orbitals; "
+                f"got {len(wires1)}"
             )
         if len(wires2) < 2:
             raise ValueError(
-                "expected at least two wires representing the unoccupied orbitals; "
-                "got {}".format(len(wires2))
+                f"expected at least two wires representing the unoccupied orbitals; "
+                f"got {len(wires2)}"
             )
 
         shape = qml.math.shape(weight)
         if shape != ():
             raise ValueError(f"Weight must be a scalar; got shape {shape}.")
 
-        self.wires1 = list(wires1)
-        self.wires2 = list(wires2)
-        wires = wires1 + wires2
+        wires1 = list(wires1)
+        wires2 = list(wires2)
 
+        self._hyperparameters = {
+            "wires1": wires1,
+            "wires2": wires2,
+        }
+
+        wires = wires1 + wires2
         super().__init__(weight, wires=wires, do_queue=do_queue, id=id)
 
-    def expand(self):
+    @property
+    def num_params(self):
+        return 1
 
-        weight = self.parameters[0]
-        s = self.wires1[0]
-        r = self.wires1[-1]
-        q = self.wires2[0]
-        p = self.wires2[-1]
+    @staticmethod
+    def compute_decomposition(
+        weight, wires, wires1, wires2
+    ):  # pylint: disable=arguments-differ,unused-argument
+        r"""Representation of the operator as a product of other operators.
+
+        .. math:: O = O_1 O_2 \dots O_n.
+
+
+
+        .. seealso:: :meth:`~.FermionicDoubleExcitation.decomposition`.
+
+        Args:
+            weight (float or tensor_like): angle :math:`\theta` entering the Z rotation
+            wires (Any or Iterable[Any]): full set of wires that the operator acts on
+            wires1 (Iterable): Wires of the qubits representing the subset of occupied orbitals
+                in the interval ``[s, r]``.
+            wires2 (Iterable): Wires of the qubits representing the subset of unoccupied
+                orbitals in the interval ``[q, p]``.
+
+        Returns:
+            list[.Operator]: decomposition of the operator
+        """
+        s = wires1[0]
+        r = wires1[-1]
+        q = wires2[0]
+        p = wires2[-1]
 
         # Sequence of the wires entering the CNOTs
-        cnots_occ = [self.wires1[l : l + 2] for l in range(len(self.wires1) - 1)]
-        cnots_unocc = [self.wires2[l : l + 2] for l in range(len(self.wires2) - 1)]
+        cnots_occ = [wires1[l : l + 2] for l in range(len(wires1) - 1)]
+        cnots_unocc = [wires2[l : l + 2] for l in range(len(wires2) - 1)]
 
         set_cnot_wires = cnots_occ + [[r, q]] + cnots_unocc
 
-        with qml.tape.QuantumTape() as tape:
+        op_list = []
 
-            _layer1(weight, s, r, q, p, set_cnot_wires)
-            _layer2(weight, s, r, q, p, set_cnot_wires)
-            _layer3(weight, s, r, q, p, set_cnot_wires)
-            _layer4(weight, s, r, q, p, set_cnot_wires)
-            _layer5(weight, s, r, q, p, set_cnot_wires)
-            _layer6(weight, s, r, q, p, set_cnot_wires)
-            _layer7(weight, s, r, q, p, set_cnot_wires)
-            _layer8(weight, s, r, q, p, set_cnot_wires)
+        op_list.extend(_layer1(weight, s, r, q, p, set_cnot_wires))
+        op_list.extend(_layer2(weight, s, r, q, p, set_cnot_wires))
+        op_list.extend(_layer3(weight, s, r, q, p, set_cnot_wires))
+        op_list.extend(_layer4(weight, s, r, q, p, set_cnot_wires))
+        op_list.extend(_layer5(weight, s, r, q, p, set_cnot_wires))
+        op_list.extend(_layer6(weight, s, r, q, p, set_cnot_wires))
+        op_list.extend(_layer7(weight, s, r, q, p, set_cnot_wires))
+        op_list.extend(_layer8(weight, s, r, q, p, set_cnot_wires))
 
-        return tape
+        return op_list
