@@ -21,7 +21,6 @@ import types
 import warnings
 
 import pennylane as qml
-from pennylane.beta import QNode
 
 
 class batch_transform:
@@ -77,11 +76,11 @@ class batch_transform:
             '''Generates two tapes, one with all RX replaced with RY,
             and the other with all RX replaced with RZ.'''
 
-            tape1 = qml.tape.JacobianTape()
-            tape2 = qml.tape.JacobianTape()
+            tape1 = qml.tape.QuantumTape()
+            tape2 = qml.tape.QuantumTape()
 
             # loop through all operations on the input tape
-            for op in tape.operations + tape.measurements:
+            for op in tape:
                 if op.name == "RX":
                     wires = op.wires
                     param = op.parameters[0]
@@ -103,15 +102,15 @@ class batch_transform:
 
     We can apply this transform to a quantum tape:
 
-    >>> with qml.tape.JacobianTape() as tape:
+    >>> with qml.tape.QuantumTape() as tape:
     ...     qml.Hadamard(wires=0)
     ...     qml.RX(-0.5, wires=0)
     ...     qml.expval(qml.PauliX(0))
     >>> tapes, fn = my_transform(tape, 0.65, 2.5)
-    >>> print(tapes[0].draw())
-     0: ──H──RY(0.325)──┤ ⟨X⟩
-    >>> print(tapes[1].draw())
-     0: ──H──RZ(1.25)──┤ ⟨X⟩
+    >>> print(qml.drawer.tape_text(tapes[0], decimals=2))
+    0: ──H──RY(0.33)─┤  <X>
+    >>> print(qml.drawer.tape_text(tapes[1], decimals=2))
+    0: ──H──RZ(1.25)─┤  <X>
 
     We can execute these tapes manually:
 
@@ -155,7 +154,8 @@ class batch_transform:
     >>> print(gradient)
     2.5800122591960153
 
-    .. UsageDetails::
+    .. details::
+        :title: Usage Details
 
         **Expansion functions**
 
@@ -271,7 +271,7 @@ class batch_transform:
         and returns a function that 'wraps' the QNode execution.
 
         The returned function should accept the same keyword arguments as
-        the QNode, and return the output of the applying the tape transform
+        the QNode, and return the output of applying the tape transform
         to the QNode's constructed tape.
         """
         transform_max_diff = tkwargs.pop("max_diff", None)
@@ -289,7 +289,7 @@ class batch_transform:
             tapes, processing_fn = self.construct(qnode.qtape, *targs, **tkwargs)
 
             interface = qnode.interface
-            execute_kwargs = getattr(qnode, "execute_kwargs", {})
+            execute_kwargs = getattr(qnode, "execute_kwargs", {}).copy()
             max_diff = execute_kwargs.pop("max_diff", 2)
             max_diff = transform_max_diff or max_diff
 
@@ -299,12 +299,6 @@ class batch_transform:
             if interface is None or not self.differentiable:
                 gradient_fn = None
 
-            elif gradient_fn in ("best", "parameter-shift"):
-                gradient_fn = qml.gradients.param_shift
-
-            elif gradient_fn == "finite-diff":
-                gradient_fn = qml.gradients.finite_diff
-
             res = qml.execute(
                 tapes,
                 device=qnode.device,
@@ -331,12 +325,12 @@ class batch_transform:
             # dev = some_transform(dev, *transform_args)
             return self._device_wrapper(*targs, **tkwargs)(qnode)
 
-        if isinstance(qnode, qml.tape.QuantumTape):
+        if isinstance(qnode, qml.tape.QuantumScript):
             # Input is a quantum tape.
             # tapes, fn = some_transform(tape, *transform_args)
             return self._tape_wrapper(*targs, **tkwargs)(qnode)
 
-        if isinstance(qnode, (qml.QNode, QNode, qml.ExpvalCost)):
+        if isinstance(qnode, (qml.QNode, qml.ExpvalCost)):
             # Input is a QNode:
             # result = some_transform(qnode, *transform_args)(*qnode_args)
             wrapper = self.qnode_wrapper(qnode, targs, tkwargs)
@@ -371,7 +365,7 @@ class batch_transform:
                 if isinstance(qnode, qml.Device):
                     return self._device_wrapper(*targs, **tkwargs)(qnode)
 
-                if isinstance(qnode, qml.tape.QuantumTape):
+                if isinstance(qnode, qml.tape.QuantumScript):
                     return self._tape_wrapper(*targs, **tkwargs)(qnode)
 
                 _wrapper = self.qnode_wrapper(qnode, targs, tkwargs)
@@ -393,7 +387,7 @@ class batch_transform:
         """Applies the batch tape transform to an input tape.
 
         Args:
-            tape (.QuantumTape): the tape to be transformed
+            tape (.QuantumScript): the tape to be transformed
             *args: positional arguments to pass to the tape transform
             **kwargs: keyword arguments to pass to the tape transform
 
@@ -409,7 +403,9 @@ class batch_transform:
         tapes, processing_fn = self.transform_fn(tape, *args, **kwargs)
 
         if processing_fn is None:
-            processing_fn = lambda x: x
+
+            def processing_fn(x):
+                return x
 
         return tapes, processing_fn
 
@@ -423,3 +419,70 @@ class batch_transform:
 
     def _tape_wrapper(self, *targs, **tkwargs):
         return lambda tape: self.construct(tape, *targs, **tkwargs)
+
+
+def map_batch_transform(transform, tapes):
+    """Map a batch transform over multiple tapes.
+
+    Args:
+        transform (.batch_transform): the batch transform
+            to be mapped
+        tapes (Sequence[QuantumScript]): The sequence of tapes the batch
+            transform should be applied to. Each tape in the sequence
+            is transformed by the batch transform.
+
+    **Example**
+
+    Consider the following tapes:
+
+    .. code-block:: python
+
+        H = qml.PauliZ(0) @ qml.PauliZ(1) - qml.PauliX(0)
+
+        with qml.tape.QuantumTape() as tape1:
+            qml.RX(0.5, wires=0)
+            qml.RY(0.1, wires=1)
+            qml.CNOT(wires=[0, 1])
+            qml.expval(H)
+
+        with qml.tape.QuantumTape() as tape2:
+            qml.Hadamard(wires=0)
+            qml.CRX(0.5, wires=[0, 1])
+            qml.CNOT(wires=[0, 1])
+            qml.expval(H + 0.5 * qml.PauliY(0))
+
+    We can use ``map_batch_transform`` to map a single
+    batch transform across both of the these tapes in such a way
+    that allows us to submit a single job for execution:
+
+    >>> tapes, fn = map_batch_transform(qml.transforms.hamiltonian_expand, [tape1, tape2])
+    >>> dev = qml.device("default.qubit", wires=2)
+    >>> fn(qml.execute(tapes, dev, qml.gradients.param_shift))
+    [0.9950041652780257, 0.8150893013179248]
+    """
+    execution_tapes = []
+    batch_fns = []
+    tape_counts = []
+
+    for t in tapes:
+        # Preprocess the tapes by applying batch transforms
+        # to each tape, and storing corresponding tapes
+        # for execution, processing functions, and list of tape lengths.
+        new_tapes, fn = transform(t)
+        execution_tapes.extend(new_tapes)
+        batch_fns.append(fn)
+        tape_counts.append(len(new_tapes))
+
+    def processing_fn(res):
+        count = 0
+        final_results = []
+
+        for idx, s in enumerate(tape_counts):
+            # apply any batch transform post-processing
+            new_res = batch_fns[idx](res[count : count + s])
+            final_results.append(new_res)
+            count += s
+
+        return final_results
+
+    return execution_tapes, processing_fn
