@@ -19,12 +19,13 @@ import numpy as np
 from random import random
 
 import pennylane as qml
+from pennylane import numpy as pnp
 from pennylane import QubitDevice, DeviceError, QuantumFunctionError
-from pennylane.operation import Sample, Variance, Expectation, Probability, State
+from pennylane.measurements import Sample, Variance, Expectation, Probability, State
 from pennylane.circuit_graph import CircuitGraph
 from pennylane.wires import Wires
 from pennylane.tape import QuantumTape
-from pennylane.measure import state
+from pennylane.measurements import state
 
 mock_qubit_device_paulis = ["PauliX", "PauliY", "PauliZ"]
 mock_qubit_device_rotations = ["RX", "RY", "RZ"]
@@ -133,7 +134,7 @@ def mock_qubit_device_with_paulis_rotations_and_methods(monkeypatch):
         m.setattr(QubitDevice, "expval", lambda self, *args, **kwargs: 0)
         m.setattr(QubitDevice, "var", lambda self, *args, **kwargs: 0)
         m.setattr(QubitDevice, "sample", lambda self, *args, **kwargs: 0)
-        m.setattr(QubitDevice, "apply", lambda self, x: None)
+        m.setattr(QubitDevice, "apply", lambda self, x, **kwargs: None)
 
         def get_qubit_device(wires=1):
             return QubitDevice(wires=wires)
@@ -141,6 +142,14 @@ def mock_qubit_device_with_paulis_rotations_and_methods(monkeypatch):
         yield get_qubit_device
 
 
+def _working_get_batch_size(tensor, expected_shape, expected_size):
+    size = QubitDevice._size(tensor)
+    if QubitDevice._ndim(tensor) > len(expected_shape) or size > expected_size:
+        return size // expected_size
+
+    return None
+
+
 class TestOperations:
     """Tests the logic related to operations"""
 
@@ -267,7 +276,7 @@ class TestObservables:
 
         with qml.tape.QuantumTape() as tape:
             qml.PauliX(wires=0)
-            qml.measure.MeasurementProcess(
+            qml.measurements.MeasurementProcess(
                 return_type="SomeUnsupportedReturnType", obs=qml.PauliZ(0)
             )
 
@@ -302,9 +311,7 @@ class TestExtractStatistics:
         """Tests that the statistics method simply builds a results list without any side-effects"""
 
         class SomeObservable(qml.operation.Observable):
-            num_params = 0
             num_wires = 1
-            par_domain = "F"
             return_type = returntype
 
         obs = SomeObservable(wires=0)
@@ -331,9 +338,7 @@ class TestExtractStatistics:
         """Tests that the statistics method returns an empty list if the return type is None"""
 
         class SomeObservable(qml.operation.Observable):
-            num_params = 0
             num_wires = 1
-            par_domain = "F"
             return_type = returntype
 
         obs = SomeObservable(wires=0)
@@ -351,9 +356,7 @@ class TestExtractStatistics:
         assert returntype not in [Expectation, Variance, Sample, Probability, State, None]
 
         class SomeObservable(qml.operation.Observable):
-            num_params = 0
             num_wires = 1
-            par_domain = "F"
             return_type = returntype
 
         obs = SomeObservable(wires=0)
@@ -370,12 +373,12 @@ class TestGenerateSamples:
         """Tests that the generate_samples method calls on its auxiliary methods correctly"""
 
         dev = mock_qubit_device()
-        number_of_states = 2 ** dev.num_wires
+        number_of_states = 2**dev.num_wires
 
         with monkeypatch.context() as m:
             # Mock the auxiliary methods such that they return the expected values
             m.setattr(QubitDevice, "sample_basis_states", lambda self, wires, b: wires)
-            m.setattr(QubitDevice, "states_to_binary", lambda a, b: (a, b))
+            m.setattr(QubitDevice, "states_to_binary", staticmethod(lambda a, b: (a, b)))
             m.setattr(QubitDevice, "analytic_probability", lambda *args: None)
             m.setattr(QubitDevice, "shots", 1000)
             dev._samples = dev.generate_samples()
@@ -419,6 +422,33 @@ class TestSampleBasisStates:
         ):
             dev.sample_basis_states(number_of_states, state_probs)
 
+    @pytest.mark.filterwarnings("ignore:Creating an ndarray from ragged nested")
+    def test_sampling_with_broadcasting(self, mock_qubit_device, monkeypatch):
+        """Tests that the sample_basis_states method samples with the correct arguments
+        when using broadcasted probabilities"""
+
+        shots = 1000
+
+        number_of_states = 4
+        dev = mock_qubit_device()
+        dev.shots = shots
+        state_probs = [[0.1, 0.2, 0.3, 0.4], [0.5, 0.2, 0.1, 0.2]]
+        # First run the sampling to see that it is using numpy.random.choice correctly
+        res = dev.sample_basis_states(number_of_states, state_probs)
+        assert qml.math.shape(res) == (2, shots)
+        assert set(res.flat).issubset({0, 1, 2, 3})
+
+        with monkeypatch.context() as m:
+            # Mock the numpy.random.choice method such that it returns the expected values
+            m.setattr("numpy.random.choice", lambda x, y, p: (x, y, p))
+            res = dev.sample_basis_states(number_of_states, state_probs)
+
+        assert len(res) == 2
+        for _res, prob in zip(res, state_probs):
+            assert np.array_equal(_res[0], np.array([0, 1, 2, 3]))
+            assert _res[1] == 1000
+            assert _res[2] == prob
+
 
 class TestStatesToBinary:
     """Test the states_to_binary method"""
@@ -428,14 +458,14 @@ class TestStatesToBinary:
         wires = 4
         shots = 10
 
-        number_of_states = 2 ** wires
+        number_of_states = 2**wires
         basis_states = np.arange(number_of_states)
         samples = np.random.choice(basis_states, shots)
 
         dev = mock_qubit_device()
         res = dev.states_to_binary(samples, wires)
 
-        format_smt = "{{:0{}b}}".format(wires)
+        format_smt = f"{{:0{wires}b}}"
         expected = np.array([[int(x) for x in list(format_smt.format(i))] for i in samples])
 
         assert np.all(res == expected)
@@ -458,6 +488,39 @@ class TestStatesToBinary:
         res = dev.states_to_binary(samples, wires)
         assert np.allclose(res, binary_states, atol=tol, rtol=0)
 
+    test_binary_conversion_data_broadcasted = [
+        (
+            np.array([[2, 3, 2, 0, 0], [3, 0, 0, 1, 1], [2, 2, 0, 1, 3]]),
+            np.array(
+                [
+                    [[1, 0], [1, 1], [1, 0], [0, 0], [0, 0]],
+                    [[1, 1], [0, 0], [0, 0], [0, 1], [0, 1]],
+                    [[1, 0], [1, 0], [0, 0], [0, 1], [1, 1]],
+                ]
+            ),
+        ),
+        (
+            np.array([[7, 7, 1, 5, 2], [3, 3, 2, 4, 6], [0, 0, 7, 2, 1]]),
+            np.array(
+                [
+                    [[1, 1, 1], [1, 1, 1], [0, 0, 1], [1, 0, 1], [0, 1, 0]],
+                    [[0, 1, 1], [0, 1, 1], [0, 1, 0], [1, 0, 0], [1, 1, 0]],
+                    [[0, 0, 0], [0, 0, 0], [1, 1, 1], [0, 1, 0], [0, 0, 1]],
+                ]
+            ),
+        ),
+    ]
+
+    @pytest.mark.parametrize("samples, binary_states", test_binary_conversion_data_broadcasted)
+    def test_correct_conversion_broadcasted(self, mock_qubit_device, samples, binary_states, tol):
+        """Tests that the states_to_binary method converts broadcasted
+        samples to binary correctly"""
+        dev = mock_qubit_device()
+        dev.shots = 5
+        wires = binary_states.shape[-1]
+        res = dev.states_to_binary(samples, wires)
+        assert np.allclose(res, binary_states, atol=tol, rtol=0)
+
 
 class TestExpval:
     """Test the expval method"""
@@ -466,7 +529,6 @@ class TestExpval:
         """Tests that expval method when the analytic attribute is True
 
         Additional QubitDevice methods that are mocked:
-        -rotate_basis
         -probability
         """
         obs = qml.PauliX(0)
@@ -480,13 +542,33 @@ class TestExpval:
             m.setattr(QubitDevice, "probability", lambda self, wires=None: probs)
             res = dev.expval(obs)
 
-        assert res == (obs.eigvals @ probs).real
+        assert res == (obs.eigvals() @ probs).real
+
+    def test_analytic_expval_broadcasted(
+        self, mock_qubit_device_with_original_statistics, monkeypatch
+    ):
+        """Tests expval method when the analytic attribute is True and using broadcasting
+
+        Additional QubitDevice methods that are mocked:
+        -probability
+        """
+        obs = qml.PauliX(0)
+        probs = np.array([[0.5, 0.5], [0.2, 0.8], [0.1, 0.9]])
+        dev = mock_qubit_device_with_original_statistics()
+
+        assert dev.shots is None
+
+        call_history = []
+        with monkeypatch.context() as m:
+            m.setattr(QubitDevice, "probability", lambda self, wires=None: probs)
+            res = dev.expval(obs)
+
+        assert np.allclose(res, (probs @ obs.eigvals()).real)
 
     def test_non_analytic_expval(self, mock_qubit_device_with_original_statistics, monkeypatch):
         """Tests that expval method when the analytic attribute is False
 
         Additional QubitDevice methods that are mocked:
-        -rotate_basis
         -sample
         -numpy.mean
         """
@@ -504,10 +586,23 @@ class TestExpval:
         assert res == obs
 
     def test_no_eigval_error(self, mock_qubit_device_with_original_statistics):
-        """Tests that an error is thrown if expval is called with an observable that does not have eigenvalues defined."""
+        """Tests that an error is thrown if expval is called with an observable that does
+        not have eigenvalues defined."""
         dev = mock_qubit_device_with_original_statistics()
-        with pytest.raises(ValueError, match="Cannot compute analytic expectations"):
-            dev.expval(qml.Hamiltonian([1.0], [qml.PauliX(0)]))
+
+        # observable with no eigenvalue representation defined
+        class MyObs(qml.operation.Observable):
+            num_wires = 1
+
+            def eigvals(self):
+                raise qml.operation.EigvalsUndefinedError
+
+        obs = MyObs(wires=0)
+
+        with pytest.raises(
+            qml.operation.EigvalsUndefinedError, match="Cannot compute analytic expectations"
+        ):
+            dev.expval(obs)
 
 
 class TestVar:
@@ -517,7 +612,6 @@ class TestVar:
         """Tests that var method when the analytic attribute is True
 
         Additional QubitDevice methods that are mocked:
-        -rotate_basis
         -probability
         """
         obs = qml.PauliX(0)
@@ -531,13 +625,33 @@ class TestVar:
             m.setattr(QubitDevice, "probability", lambda self, wires=None: probs)
             res = dev.var(obs)
 
-        assert res == (obs.eigvals ** 2) @ probs - (obs.eigvals @ probs).real ** 2
+        assert res == (obs.eigvals() ** 2) @ probs - (obs.eigvals() @ probs).real ** 2
+
+    def test_analytic_var_broadcasted(
+        self, mock_qubit_device_with_original_statistics, monkeypatch
+    ):
+        """Tests var method when the analytic attribute is True and using broadcasting
+
+        Additional QubitDevice methods that are mocked:
+        -probability
+        """
+        obs = qml.PauliX(0)
+        probs = [0.5, 0.5]
+        dev = mock_qubit_device_with_original_statistics()
+
+        assert dev.shots is None
+
+        call_history = []
+        with monkeypatch.context() as m:
+            m.setattr(QubitDevice, "probability", lambda self, wires=None: probs)
+            res = dev.var(obs)
+
+        assert np.allclose(res, probs @ (obs.eigvals() ** 2) - (probs @ obs.eigvals()).real ** 2)
 
     def test_non_analytic_var(self, mock_qubit_device_with_original_statistics, monkeypatch):
         """Tests that var method when the analytic attribute is False
 
         Additional QubitDevice methods that are mocked:
-        -rotate_basis
         -sample
         -numpy.var
         """
@@ -557,8 +671,20 @@ class TestVar:
     def test_no_eigval_error(self, mock_qubit_device_with_original_statistics):
         """Tests that an error is thrown if var is called with an observable that does not have eigenvalues defined."""
         dev = mock_qubit_device_with_original_statistics()
-        with pytest.raises(ValueError, match="Cannot compute analytic variance"):
-            dev.var(qml.Hamiltonian([1.0], [qml.PauliX(0)]))
+
+        # observable with no eigenvalue representation defined
+        class MyObs(qml.operation.Observable):
+            num_wires = 1
+
+            def eigvals(self):
+                raise qml.operation.EigvalsUndefinedError
+
+        obs = MyObs(wires=0)
+
+        with pytest.raises(
+            qml.operation.EigvalsUndefinedError, match="Cannot compute analytic variance"
+        ):
+            dev.var(obs)
 
 
 class TestSample:
@@ -567,7 +693,7 @@ class TestSample:
     def test_only_ones_minus_ones(
         self, mock_qubit_device_with_original_statistics, monkeypatch, tol
     ):
-        """Test that pauli_eigvals_as_samples method only produces -1 and 1 samples"""
+        """Test that sample for a single Pauli observable only produces -1 and 1 samples"""
         obs = qml.PauliX(0)
         dev = mock_qubit_device_with_original_statistics()
         dev._samples = np.array([[1, 0], [0, 0]])
@@ -575,12 +701,13 @@ class TestSample:
         with monkeypatch.context() as m:
             res = dev.sample(obs)
 
-        assert np.allclose(res ** 2, 1, atol=tol, rtol=0)
+        assert np.shape(res) == (2,)
+        assert np.allclose(res**2, 1, atol=tol, rtol=0)
 
     def test_correct_custom_eigenvalues(
         self, mock_qubit_device_with_original_statistics, monkeypatch, tol
     ):
-        """Test that pauli_eigvals_as_samples method only produces samples of eigenvalues"""
+        """Test that sample for a product of Pauli observables produces samples of eigenvalues"""
         obs = qml.PauliX(0) @ qml.PauliZ(1)
         dev = mock_qubit_device_with_original_statistics(wires=2)
         dev._samples = np.array([[1, 0], [0, 0]])
@@ -595,7 +722,7 @@ class TestSample:
     ):
         """Test that when we sample a device without providing an observable or wires then it
         will return the raw samples"""
-        obs = qml.measure.sample(op=None, wires=None)
+        obs = qml.measurements.sample(op=None, wires=None)
         dev = mock_qubit_device_with_original_statistics(wires=2)
         generated_samples = np.array([[1, 0], [1, 1]])
         dev._samples = generated_samples
@@ -608,7 +735,7 @@ class TestSample:
     ):
         """Test that when we sample a device without providing an observable but we specify
         wires then it returns the generated samples for only those wires"""
-        obs = qml.measure.sample(op=None, wires=[1])
+        obs = qml.measurements.sample(op=None, wires=[1])
         dev = mock_qubit_device_with_original_statistics(wires=2)
         generated_samples = np.array([[1, 0], [1, 1]])
         dev._samples = generated_samples
@@ -619,10 +746,80 @@ class TestSample:
         assert np.array_equal(res, wire_samples)
 
     def test_no_eigval_error(self, mock_qubit_device_with_original_statistics):
-        """Tests that an error is thrown if sample is called with an observable that does not have eigenvalues defined."""
+        """Tests that an error is thrown if sample is called with an observable
+        that does not have eigenvalues defined."""
         dev = mock_qubit_device_with_original_statistics()
         dev._samples = np.array([[1, 0], [0, 0]])
-        with pytest.raises(ValueError, match="Cannot compute samples"):
+        with pytest.raises(qml.operation.EigvalsUndefinedError, match="Cannot compute samples"):
+            dev.sample(qml.Hamiltonian([1.0], [qml.PauliX(0)]))
+
+
+class TestSampleWithBroadcasting:
+    """Test the sample method when broadcasting is used"""
+
+    def test_only_ones_minus_ones(
+        self, mock_qubit_device_with_original_statistics, monkeypatch, tol
+    ):
+        """Test that sample for a single Pauli observable only produces -1 and 1 samples
+        when using broadcasting"""
+        obs = qml.PauliX(0)
+        dev = mock_qubit_device_with_original_statistics()
+        dev._samples = np.array([[[0, 0], [0, 0]], [[1, 0], [1, 0]], [[0, 0], [1, 0]]])
+
+        with monkeypatch.context() as m:
+            res = dev.sample(obs)
+
+        assert np.allclose(res, [[1, 1], [-1, -1], [1, -1]], atol=tol, rtol=0)
+
+    def test_correct_custom_eigenvalues(
+        self, mock_qubit_device_with_original_statistics, monkeypatch, tol
+    ):
+        """Test that sample for a product of Pauli observables produces samples
+        of eigenvalues when using broadcasting"""
+        obs = qml.PauliX(0) @ qml.PauliZ(1)
+        dev = mock_qubit_device_with_original_statistics(wires=2)
+        dev._samples = np.array([[1, 0], [0, 0]])
+        dev._samples = np.array([[[1, 0], [0, 0]], [[0, 1], [1, 1]], [[1, 0], [0, 1]]])
+
+        with monkeypatch.context() as m:
+            res = dev.sample(obs)
+
+        assert np.array_equal(res, np.array([[-1, 1], [-1, 1], [-1, -1]]))
+
+    def test_sample_with_no_observable_and_no_wires(
+        self, mock_qubit_device_with_original_statistics, tol
+    ):
+        """Test that when we sample a device without providing an observable or wires then it
+        will return the raw samples when using broadcasting"""
+        obs = qml.measurements.sample(op=None, wires=None)
+        dev = mock_qubit_device_with_original_statistics(wires=2)
+        generated_samples = np.array([[[1, 0], [1, 1]], [[1, 1], [0, 0]], [[0, 1], [1, 0]]])
+        dev._samples = generated_samples
+
+        res = dev.sample(obs)
+        assert np.array_equal(res, generated_samples)
+
+    def test_sample_with_no_observable_and_with_wires(
+        self, mock_qubit_device_with_original_statistics, tol
+    ):
+        """Test that when we sample a device without providing an observable but we specify wires
+        then it returns the generated samples for only those wires when using broadcasting"""
+        obs = qml.measurements.sample(op=None, wires=[1])
+        dev = mock_qubit_device_with_original_statistics(wires=2)
+        generated_samples = np.array([[[1, 0], [1, 1]], [[1, 1], [0, 0]], [[0, 1], [1, 0]]])
+        dev._samples = generated_samples
+
+        wire_samples = np.array([[[0], [1]], [[1], [0]], [[1], [0]]])
+        res = dev.sample(obs)
+
+        assert np.array_equal(res, wire_samples)
+
+    def test_no_eigval_error(self, mock_qubit_device_with_original_statistics):
+        """Tests that an error is thrown if sample is called with an observable
+        that does not have eigenvalues defined when using broadcasting."""
+        dev = mock_qubit_device_with_original_statistics()
+        dev._samples = np.array([[[1, 0], [1, 1]], [[1, 1], [0, 0]], [[0, 1], [1, 0]]])
+        with pytest.raises(qml.operation.EigvalsUndefinedError, match="Cannot compute samples"):
             dev.sample(qml.Hamiltonian([1.0], [qml.PauliX(0)]))
 
 
@@ -635,17 +832,114 @@ class TestEstimateProb:
     def test_estimate_probability(
         self, wires, expected, mock_qubit_device_with_original_statistics, monkeypatch
     ):
-        """Tests probability method when the analytic attribute is True."""
+        """Tests the estimate_probability method"""
         dev = mock_qubit_device_with_original_statistics(wires=2)
         samples = np.array([[0, 0], [1, 1], [1, 1], [0, 0]])
 
         with monkeypatch.context() as m:
             m.setattr(dev, "_samples", samples)
-            m.setattr(dev, "shots", 4)
             res = dev.estimate_probability(wires=wires)
 
         assert np.allclose(res, expected)
 
+    @pytest.mark.parametrize(
+        "wires, expected",
+        [
+            ([0], [[0.0, 0.5], [1.0, 0.5]]),
+            (None, [[0.0, 0.5], [0, 0], [0, 0.5], [1.0, 0]]),
+            ([0, 1], [[0.0, 0.5], [0, 0], [0, 0.5], [1.0, 0]]),
+        ],
+    )
+    def test_estimate_probability_with_binsize(
+        self, wires, expected, mock_qubit_device_with_original_statistics, monkeypatch
+    ):
+        """Tests the estimate_probability method with a bin size"""
+        dev = mock_qubit_device_with_original_statistics(wires=2)
+        samples = np.array([[1, 1], [1, 1], [1, 0], [0, 0]])
+        bin_size = 2
+
+        with monkeypatch.context() as m:
+            m.setattr(dev, "_samples", samples)
+            res = dev.estimate_probability(wires=wires, bin_size=bin_size)
+
+        assert np.allclose(res, expected)
+
+    @pytest.mark.parametrize(
+        "wires, expected",
+        [
+            ([0], [[0.0, 1.0], [0.5, 0.5], [0.25, 0.75]]),
+            (None, [[0, 0, 0.25, 0.75], [0.5, 0, 0, 0.5], [0.25, 0, 0.25, 0.5]]),
+            ([0, 1], [[0, 0, 0.25, 0.75], [0.5, 0, 0, 0.5], [0.25, 0, 0.25, 0.5]]),
+        ],
+    )
+    def test_estimate_probability_with_broadcasting(
+        self, wires, expected, mock_qubit_device_with_original_statistics, monkeypatch
+    ):
+        """Tests the estimate_probability method with parameter broadcasting"""
+        dev = mock_qubit_device_with_original_statistics(wires=2)
+        samples = np.array(
+            [
+                [[1, 0], [1, 1], [1, 1], [1, 1]],
+                [[0, 0], [1, 1], [1, 1], [0, 0]],
+                [[1, 0], [1, 1], [1, 1], [0, 0]],
+            ]
+        )
+
+        with monkeypatch.context() as m:
+            m.setattr(dev, "_samples", samples)
+            res = dev.estimate_probability(wires=wires)
+
+        assert np.allclose(res, expected)
+
+    @pytest.mark.parametrize(
+        "wires, expected",
+        [
+            (
+                [0],
+                [
+                    [[0, 0, 0.5], [1, 1, 0.5]],
+                    [[0.5, 0.5, 0], [0.5, 0.5, 1]],
+                    [[0, 0.5, 1], [1, 0.5, 0]],
+                ],
+            ),
+            (
+                None,
+                [
+                    [[0, 0, 0], [0, 0, 0.5], [0.5, 0, 0], [0.5, 1, 0.5]],
+                    [[0.5, 0.5, 0], [0, 0, 0], [0, 0, 0], [0.5, 0.5, 1]],
+                    [[0, 0.5, 0.5], [0, 0, 0.5], [0.5, 0, 0], [0.5, 0.5, 0]],
+                ],
+            ),
+            (
+                [0, 1],
+                [
+                    [[0, 0, 0], [0, 0, 0.5], [0.5, 0, 0], [0.5, 1, 0.5]],
+                    [[0.5, 0.5, 0], [0, 0, 0], [0, 0, 0], [0.5, 0.5, 1]],
+                    [[0, 0.5, 0.5], [0, 0, 0.5], [0.5, 0, 0], [0.5, 0.5, 0]],
+                ],
+            ),
+        ],
+    )
+    def test_estimate_probability_with_binsize_with_broadcasting(
+        self, wires, expected, mock_qubit_device_with_original_statistics, monkeypatch
+    ):
+        """Tests the estimate_probability method with a bin size and parameter broadcasting"""
+        dev = mock_qubit_device_with_original_statistics(wires=2)
+        bin_size = 2
+        samples = np.array(
+            [
+                [[1, 0], [1, 1], [1, 1], [1, 1], [1, 1], [0, 1]],
+                [[0, 0], [1, 1], [1, 1], [0, 0], [1, 1], [1, 1]],
+                [[1, 0], [1, 1], [1, 1], [0, 0], [0, 1], [0, 0]],
+            ]
+        )
+
+        with monkeypatch.context() as m:
+            m.setattr(dev, "_samples", samples)
+            res = dev.estimate_probability(wires=wires, bin_size=bin_size)
+
+        assert np.allclose(res, expected)
+
 
 class TestMarginalProb:
     """Test the marginal_prob method"""
@@ -671,7 +965,7 @@ class TestMarginalProb:
         """Test that the correct arguments are passed to the marginal_prob method"""
 
         # Generate probabilities
-        probs = np.array([random() for i in range(2 ** 3)])
+        probs = np.array([random() for i in range(2**3)])
         probs /= sum(probs)
 
         spy = mocker.spy(np, "sum")
@@ -726,6 +1020,58 @@ class TestMarginalProb:
         res = dev.marginal_prob(probs, wires=None)
         assert np.allclose(res, probs, atol=tol, rtol=0)
 
+    # Note that the broadcasted probs enter `marginal_probs` as a flattened array
+    broadcasted_marginal_test_data = [
+        (
+            np.array([[0.1, 0.2, 0.3, 0.4], [0.8, 0.02, 0.05, 0.13], [0.6, 0.3, 0.02, 0.08]]),
+            np.array([[0.4, 0.6], [0.85, 0.15], [0.62, 0.38]]),
+            [1],
+            2,
+        ),
+        (
+            np.array(
+                [
+                    [0.17, 0.06, 0.21, 0.04, 0.19, 0.19, 0.08, 0.06],
+                    [0.07, 0.04, 0.11, 0.04, 0.29, 0.04, 0.18, 0.23],
+                ]
+            ),
+            np.array([[0.38, 0.27, 0.1, 0.25], [0.18, 0.47, 0.08, 0.27]]),
+            [2, 0],
+            3,
+        ),
+    ]
+
+    @pytest.mark.parametrize("probs, marginals, wires, num_wires", broadcasted_marginal_test_data)
+    def test_correct_broadcasted_marginals_returned(
+        self,
+        monkeypatch,
+        mock_qubit_device_with_original_statistics,
+        probs,
+        marginals,
+        wires,
+        num_wires,
+        tol,
+    ):
+        """Test that the correct marginals are returned by the marginal_prob method when
+        broadcasting is used"""
+        dev = mock_qubit_device_with_original_statistics(num_wires)
+        with monkeypatch.context() as m:
+            m.setattr(dev, "_get_batch_size", _working_get_batch_size)
+            res = dev.marginal_prob(probs, wires=wires)
+
+        assert np.allclose(res, marginals, atol=tol, rtol=0)
+
+    @pytest.mark.parametrize("probs, marginals, wires, num_wires", broadcasted_marginal_test_data)
+    def test_correct_broadcasted_marginals_returned_wires_none(
+        self, mock_qubit_device_with_original_statistics, probs, marginals, wires, num_wires, tol
+    ):
+        """Test that the correct marginals are returned by the marginal_prob method when
+        broadcasting is used"""
+        dev = mock_qubit_device_with_original_statistics(num_wires)
+
+        res = dev.marginal_prob(probs, wires=None)
+        assert np.allclose(res, probs.reshape((-1, 2**num_wires)), atol=tol, rtol=0)
+
 
 class TestActiveWires:
     """Test that the active_wires static method works as required."""
@@ -750,6 +1096,7 @@ class TestCapabilities:
             "supports_finite_shots": True,
             "supports_tensor_observables": True,
             "returns_probs": True,
+            "supports_broadcasting": False,
         }
         assert capabilities == QubitDevice.capabilities()
 
@@ -805,6 +1152,59 @@ class TestExecution:
         assert dev_1.num_executions == num_evals_1 + num_evals_3
 
 
+class TestExecutionBroadcasted:
+    """Tests for the execute method with broadcasted parameters"""
+
+    def test_device_executions(self):
+        """Test the number of times a qubit device is executed over a QNode's
+        lifetime is tracked by `num_executions`"""
+
+        dev_1 = qml.device("default.qubit", wires=2)
+
+        def circuit_1(x, y):
+            qml.RX(x, wires=[0])
+            qml.RY(y, wires=[1])
+            qml.CNOT(wires=[0, 1])
+            return qml.expval(qml.PauliZ(0) @ qml.PauliX(1))
+
+        node_1 = qml.QNode(circuit_1, dev_1)
+        num_evals_1 = 10
+
+        for _ in range(num_evals_1):
+            node_1(0.432, np.array([0.12, 0.5, 3.2]))
+        assert dev_1.num_executions == num_evals_1
+
+        # test a second instance of a default qubit device
+        dev_2 = qml.device("default.qubit", wires=2)
+
+        assert dev_2.num_executions == 0
+
+        def circuit_2(x, y):
+            qml.RX(x, wires=[0])
+            qml.CNOT(wires=[0, 1])
+            return qml.expval(qml.PauliZ(0) @ qml.PauliX(1))
+
+        node_2 = qml.QNode(circuit_2, dev_2)
+        num_evals_2 = 5
+
+        for _ in range(num_evals_2):
+            node_2(np.array([0.432, 0.61, 8.2]), 0.12)
+        assert dev_2.num_executions == num_evals_2
+
+        # test a new circuit on an existing instance of a qubit device
+        def circuit_3(x, y):
+            qml.RY(y, wires=[1])
+            qml.CNOT(wires=[0, 1])
+            return qml.expval(qml.PauliZ(0) @ qml.PauliX(1))
+
+        node_3 = qml.QNode(circuit_3, dev_1)
+        num_evals_3 = 7
+
+        for _ in range(num_evals_3):
+            node_3(np.array([0.432, 0.2]), np.array([0.12, 1.214]))
+        assert dev_1.num_executions == num_evals_1 + num_evals_3
+
+
 class TestBatchExecution:
     """Tests for the batch_execute method."""
 
@@ -812,7 +1212,7 @@ class TestBatchExecution:
         qml.PauliX(wires=0)
         qml.expval(qml.PauliZ(wires=0)), qml.expval(qml.PauliZ(wires=1))
 
-    with qml.tape.JacobianTape() as tape2:
+    with qml.tape.QuantumTape() as tape2:
         qml.PauliX(wires=0)
         qml.expval(qml.PauliZ(wires=0))
 
@@ -841,10 +1241,12 @@ class TestBatchExecution:
 
         assert spy.call_count == n_tapes
 
-    def test_result(self, mock_qubit_device_with_paulis_and_methods, tol):
+    @pytest.mark.parametrize("r_dtype", [np.float32, np.float64])
+    def test_result(self, mock_qubit_device_with_paulis_and_methods, r_dtype, tol):
         """Tests that the result has the correct shape and entry types."""
 
         dev = mock_qubit_device_with_paulis_and_methods(wires=2)
+        dev.R_DTYPE = r_dtype
 
         tapes = [self.tape1, self.tape2]
         res = dev.batch_execute(tapes)
@@ -852,6 +1254,8 @@ class TestBatchExecution:
         assert len(res) == 2
         assert np.allclose(res[0], dev.execute(self.tape1), rtol=tol, atol=0)
         assert np.allclose(res[1], dev.execute(self.tape2), rtol=tol, atol=0)
+        assert res[0].dtype == r_dtype
+        assert res[1].dtype == r_dtype
 
     def test_result_empty_tape(self, mock_qubit_device_with_paulis_and_methods, tol):
         """Tests that the result has the correct shape and entry types for empty tapes."""
@@ -910,6 +1314,7 @@ class TestShotList:
         [[(10, 3)], [(10, 3)], (3, 2), 30],
     ]
 
+    @pytest.mark.autograd
     @pytest.mark.parametrize("shot_list,shot_vector,expected_shape,total_shots", shot_data)
     def test_multiple_expval(self, shot_list, shot_vector, expected_shape, total_shots):
         """Test multiple expectation values"""
@@ -929,8 +1334,10 @@ class TestShotList:
         assert circuit.device.shots == total_shots
 
         # test gradient works
-        res = qml.jacobian(circuit)(0.5, 0.1)
-        assert res.shape == (2,) + expected_shape
+        res = qml.jacobian(circuit)(*pnp.array([0.5, 0.1], requires_grad=True))
+        assert isinstance(res, tuple) and len(res) == 2
+        assert res[0].shape == expected_shape
+        assert res[1].shape == expected_shape
 
     shot_data = [
         [[1, 2, 3, 10], [(1, 1), (2, 1), (3, 1), (10, 1)], (4, 4), 16],
@@ -944,6 +1351,7 @@ class TestShotList:
         [[(10, 3)], [(10, 3)], (3, 4), 30],
     ]
 
+    @pytest.mark.autograd
     @pytest.mark.parametrize("shot_list,shot_vector,expected_shape,total_shots", shot_data)
     def test_probs(self, shot_list, shot_vector, expected_shape, total_shots):
         """Test a probability return"""
@@ -963,7 +1371,7 @@ class TestShotList:
         assert circuit.device.shots == total_shots
 
         # test gradient works
-        res = qml.jacobian(circuit)(0.5, 0.1)
+        res = qml.jacobian(circuit, argnum=[0, 1])(0.5, 0.1)
 
     shot_data = [
         [[1, 2, 3, 10], [(1, 1), (2, 1), (3, 1), (10, 1)], (4, 2, 2), 16],
@@ -977,6 +1385,7 @@ class TestShotList:
         [[(10, 3)], [(10, 3)], (3, 2, 2), 30],
     ]
 
+    @pytest.mark.autograd
     @pytest.mark.parametrize("shot_list,shot_vector,expected_shape,total_shots", shot_data)
     def test_multiple_probs(self, shot_list, shot_vector, expected_shape, total_shots):
         """Test multiple probability returns"""
@@ -996,7 +1405,7 @@ class TestShotList:
         assert circuit.device.shots == total_shots
 
         # test gradient works
-        res = qml.jacobian(circuit)(0.5, 0.1)
+        res = qml.jacobian(circuit, argnum=[0, 1])(0.5, 0.1)
 
     shot_data = [
         [[1, 2, 3, 10], [(1, 1), (2, 1), (3, 1), (10, 1)], [(), (2,), (3,), (10,)], 16],
@@ -1069,3 +1478,55 @@ class TestShotList:
 
         with pytest.raises(ValueError, match="Unknown shot sequence"):
             qml.device("default.qubit", wires=2, shots=["a", "b", "c"])
+
+    @pytest.mark.parametrize("shot_list", [[1, 2, 3, 10], [1, 2, 2, 10, 1, 1], [10, 10, 10], [1]])
+    def test_error_shot_list_with_broadcasting(self, shot_list):
+        """Test that an error is raised when using parameter broadcasting with a
+        shot vector is attempted."""
+        dev = qml.device("default.qubit", wires=2, shots=shot_list)
+
+        @qml.qnode(dev)
+        def circuit():
+            qml.RY(np.zeros(3), wires=0)
+            qml.CNOT(wires=[0, 1])
+            return qml.sample(qml.PauliZ(wires=0)), qml.sample(qml.PauliZ(wires=1))
+
+        with pytest.raises(NotImplementedError, match="Parameter broadcasting when using"):
+            circuit()
+
+
+class TestGetBatchSize:
+    """Tests for the helper method ``_get_batch_size`` of ``QubitDevice``."""
+
+    @pytest.mark.parametrize("shape", [(4, 4), (1, 8), (4,)])
+    def test_batch_size_always_None(self, mock_qubit_device, shape):
+        """Test that QubitDevice always reports a batch_size of None."""
+        dev = mock_qubit_device()
+        tensor0 = np.ones(shape, dtype=complex)
+        assert dev._get_batch_size(tensor0, shape, qml.math.prod(shape)) is None
+        tensor1 = np.arange(np.prod(shape)).reshape(shape)
+        assert dev._get_batch_size(tensor1, shape, qml.math.prod(shape)) is None
+
+        broadcasted_shape = (1,) + shape
+        tensor0 = np.ones(broadcasted_shape, dtype=complex)
+        assert (
+            dev._get_batch_size(tensor0, broadcasted_shape, qml.math.prod(broadcasted_shape))
+            is None
+        )
+        tensor1 = np.arange(np.prod(broadcasted_shape)).reshape(broadcasted_shape)
+        assert (
+            dev._get_batch_size(tensor1, broadcasted_shape, qml.math.prod(broadcasted_shape))
+            is None
+        )
+
+        broadcasted_shape = (3,) + shape
+        tensor0 = np.ones(broadcasted_shape, dtype=complex)
+        assert (
+            dev._get_batch_size(tensor0, broadcasted_shape, qml.math.prod(broadcasted_shape))
+            is None
+        )
+        tensor1 = np.arange(np.prod(broadcasted_shape)).reshape(broadcasted_shape)
+        assert (
+            dev._get_batch_size(tensor1, broadcasted_shape, qml.math.prod(broadcasted_shape))
+            is None
+        )
