@@ -22,22 +22,6 @@ from pennylane import numpy as pnp
 from pennylane.wires import Wires
 from pennylane.devices import DefaultQubit, DefaultMixed
 
-try:
-    import torch
-except ImportError as e:
-    pass
-
-
-try:
-    import tensorflow as tf
-
-    if tf.__version__[0] == "1":
-        tf.enable_eager_execution()
-
-    from tensorflow import Variable
-except ImportError as e:
-    pass
-
 
 @pytest.fixture(scope="function")
 def seed():
@@ -45,6 +29,14 @@ def seed():
     np.random.seed(0)
 
 
+def catch_warn_ExpvalCost(ansatz, hamiltonian, device, **kwargs):
+    """Computes the ExpvalCost and catches the initial deprecation warning."""
+
+    with pytest.warns(UserWarning, match="is deprecated,"):
+        res = qml.ExpvalCost(ansatz, hamiltonian, device, **kwargs)
+    return res
+
+
 #####################################################
 # Hamiltonians
 
@@ -192,7 +184,7 @@ ANSAETZE = [
 
 EMPTY_PARAMS = []
 VAR_PARAMS = [0.5]
-EMBED_PARAMS = np.array([1 / np.sqrt(2 ** 3)] * 2 ** 3)
+EMBED_PARAMS = np.array([1 / np.sqrt(2**3)] * 2**3)
 LAYER_PARAMS = np.random.random(qml.templates.StronglyEntanglingLayers.shape(n_layers=2, n_wires=3))
 
 CIRCUITS = [
@@ -333,7 +325,7 @@ class TestVQE:
         """Tests that the cost function evaluates properly"""
         hamiltonian = qml.Hamiltonian(coeffs, observables)
         dev = qml.device("default.qubit", wires=3)
-        expval = qml.ExpvalCost(ansatz, hamiltonian, dev)
+        expval = catch_warn_ExpvalCost(ansatz, hamiltonian, dev)
         assert type(expval(params)) == np.float64
         assert np.shape(expval(params)) == ()  # expval should be scalar
 
@@ -344,7 +336,7 @@ class TestVQE:
         """Tests that the cost function returns correct expectation values"""
         dev = qml.device("default.qubit", wires=2)
         hamiltonian = qml.Hamiltonian(coeffs, observables)
-        cost = qml.ExpvalCost(lambda params, **kwargs: None, hamiltonian, dev)
+        cost = catch_warn_ExpvalCost(lambda params, **kwargs: None, hamiltonian, dev)
         assert cost([]) == sum(expected)
 
     @pytest.mark.parametrize("ansatz", JUNK_INPUTS)
@@ -352,8 +344,9 @@ class TestVQE:
         """Tests that the cost function raises an exception if the ansatz is not valid"""
         hamiltonian = qml.Hamiltonian((1.0,), [qml.PauliZ(0)])
         with pytest.raises(ValueError, match="not a callable function."):
-            cost = qml.ExpvalCost(4, hamiltonian, mock_device())
+            cost = catch_warn_ExpvalCost(4, hamiltonian, mock_device())
 
+    @pytest.mark.autograd
     @pytest.mark.parametrize("coeffs, observables, expected", hamiltonians_with_expvals)
     def test_passing_kwargs(self, coeffs, observables, expected):
         """Test that the step size and order used for the finite differences
@@ -361,40 +354,40 @@ class TestVQE:
         keyword arguments."""
         dev = qml.device("default.qubit", wires=2)
         hamiltonian = qml.Hamiltonian(coeffs, observables)
-        cost = qml.ExpvalCost(lambda params, **kwargs: None, hamiltonian, dev, h=123, order=2)
+        cost = catch_warn_ExpvalCost(
+            lambda params, **kwargs: None, hamiltonian, dev, h=123, order=2
+        )
 
         # Checking that the qnodes contain the step size and order
         for qnode in cost.qnodes:
-            assert qnode.diff_options["h"] == 123
-            assert qnode.diff_options["order"] == 2
+            assert qnode.gradient_kwargs["h"] == 123
+            assert qnode.gradient_kwargs["order"] == 2
 
+    @pytest.mark.torch
     @pytest.mark.slow
-    @pytest.mark.parametrize("interface", ["tf", "torch", "autograd"])
-    def test_optimize(self, interface, tf_support, torch_support):
+    @pytest.mark.parametrize("shots", [None, [(8000, 5)], [(8000, 5), (9000, 4)]])
+    def test_optimize_torch(self, shots):
         """Test that an ExpvalCost with observable optimization gives the same result as another
         ExpvalCost without observable optimization."""
-        if interface == "tf" and not tf_support:
-            pytest.skip("This test requires TensorFlow")
-        if interface == "torch" and not torch_support:
-            pytest.skip("This test requires Torch")
+        import torch
 
-        dev = qml.device("default.qubit", wires=4)
+        dev = qml.device("default.qubit", wires=4, shots=shots)
         hamiltonian = big_hamiltonian
 
-        cost = qml.ExpvalCost(
+        cost = catch_warn_ExpvalCost(
             qml.templates.StronglyEntanglingLayers,
             hamiltonian,
             dev,
             optimize=True,
-            interface=interface,
+            interface="torch",
             diff_method="parameter-shift",
         )
-        cost2 = qml.ExpvalCost(
+        cost2 = catch_warn_ExpvalCost(
             qml.templates.StronglyEntanglingLayers,
             hamiltonian,
             dev,
             optimize=False,
-            interface=interface,
+            interface="torch",
             diff_method="parameter-shift",
         )
 
@@ -412,17 +405,99 @@ class TestVQE:
         assert exec_opt == 5  # Number of groups in the Hamiltonian
         assert exec_no_opt == 15
 
-        assert np.allclose(c1, c2)
+        assert np.allclose(c1, c2, atol=1e-1)
 
-    @pytest.mark.parametrize("interface", ["tf", "torch", "autograd"])
-    def test_optimize_multiple_terms(self, interface, tf_support, torch_support):
+    @pytest.mark.tf
+    @pytest.mark.slow
+    @pytest.mark.parametrize("shots", [None, [(8000, 5)], [(8000, 5), (9000, 4)]])
+    def test_optimize_tf(self, shots):
+        """Test that an ExpvalCost with observable optimization gives the same result as another
+        ExpvalCost without observable optimization."""
+
+        dev = qml.device("default.qubit", wires=4, shots=shots)
+        hamiltonian = big_hamiltonian
+
+        cost = catch_warn_ExpvalCost(
+            qml.templates.StronglyEntanglingLayers,
+            hamiltonian,
+            dev,
+            optimize=True,
+            interface="tf",
+            diff_method="parameter-shift",
+        )
+        cost2 = catch_warn_ExpvalCost(
+            qml.templates.StronglyEntanglingLayers,
+            hamiltonian,
+            dev,
+            optimize=False,
+            interface="tf",
+            diff_method="parameter-shift",
+        )
+
+        np.random.seed(1967)
+        shape = qml.templates.StronglyEntanglingLayers.shape(n_layers=2, n_wires=4)
+        w = np.random.random(shape)
+
+        c1 = cost(w)
+        exec_opt = dev.num_executions
+        dev._num_executions = 0
+
+        c2 = cost2(w)
+        exec_no_opt = dev.num_executions
+
+        assert exec_opt == 5  # Number of groups in the Hamiltonian
+        assert exec_no_opt == 15
+
+        assert np.allclose(c1, c2, atol=1e-1)
+
+    @pytest.mark.autograd
+    @pytest.mark.slow
+    @pytest.mark.parametrize("shots", [None, [(8000, 5)], [(8000, 5), (9000, 4)]])
+    def test_optimize_autograd(self, shots):
+        """Test that an ExpvalCost with observable optimization gives the same result as another
+        ExpvalCost without observable optimization."""
+
+        dev = qml.device("default.qubit", wires=4, shots=shots)
+        hamiltonian = big_hamiltonian
+
+        cost = catch_warn_ExpvalCost(
+            qml.templates.StronglyEntanglingLayers,
+            hamiltonian,
+            dev,
+            optimize=True,
+            interface="autograd",
+            diff_method="parameter-shift",
+        )
+        cost2 = catch_warn_ExpvalCost(
+            qml.templates.StronglyEntanglingLayers,
+            hamiltonian,
+            dev,
+            optimize=False,
+            interface="autograd",
+            diff_method="parameter-shift",
+        )
+
+        np.random.seed(1967)
+        shape = qml.templates.StronglyEntanglingLayers.shape(n_layers=2, n_wires=4)
+        w = np.random.random(shape)
+
+        c1 = cost(w)
+        exec_opt = dev.num_executions
+        dev._num_executions = 0
+
+        c2 = cost2(w)
+        exec_no_opt = dev.num_executions
+
+        assert exec_opt == 5  # Number of groups in the Hamiltonian
+        assert exec_no_opt == 15
+
+        assert np.allclose(c1, c2, atol=1e-1)
+
+    @pytest.mark.autograd
+    def test_optimize_multiple_terms_autograd(self):
         """Test that an ExpvalCost with observable optimization gives the same
         result as another ExpvalCost without observable optimization even when there
         are non-unique Hamiltonian terms."""
-        if interface == "tf" and not tf_support:
-            pytest.skip("This test requires TensorFlow")
-        if interface == "torch" and not torch_support:
-            pytest.skip("This test requires Torch")
 
         dev = qml.device("default.qubit", wires=5)
         obs = [
@@ -439,20 +514,20 @@ class TestVQE:
         coefs = (np.random.rand(len(obs)) - 0.5) * 2
         hamiltonian = qml.Hamiltonian(coefs, obs)
 
-        cost = qml.ExpvalCost(
+        cost = catch_warn_ExpvalCost(
             qml.templates.StronglyEntanglingLayers,
             hamiltonian,
             dev,
             optimize=True,
-            interface=interface,
+            interface="autograd",
             diff_method="parameter-shift",
         )
-        cost2 = qml.ExpvalCost(
+        cost2 = catch_warn_ExpvalCost(
             qml.templates.StronglyEntanglingLayers,
             hamiltonian,
             dev,
             optimize=False,
-            interface=interface,
+            interface="autograd",
             diff_method="parameter-shift",
         )
 
@@ -472,20 +547,129 @@ class TestVQE:
 
         assert np.allclose(c1, c2)
 
+    @pytest.mark.torch
+    def test_optimize_multiple_terms_torch(self):
+        """Test that an ExpvalCost with observable optimization gives the same
+        result as another ExpvalCost without observable optimization even when there
+        are non-unique Hamiltonian terms."""
+
+        dev = qml.device("default.qubit", wires=5)
+        obs = [
+            qml.PauliZ(wires=[2]) @ qml.PauliZ(wires=[4]),  # <---- These two terms
+            qml.PauliZ(wires=[4]) @ qml.PauliZ(wires=[2]),  # <---- are equal
+            qml.PauliZ(wires=[1]),
+            qml.PauliZ(wires=[2]),
+            qml.PauliZ(wires=[1]) @ qml.PauliZ(wires=[2]),
+            qml.PauliZ(wires=[2]) @ qml.PauliZ(wires=[0]),
+            qml.PauliZ(wires=[3]) @ qml.PauliZ(wires=[1]),
+            qml.PauliZ(wires=[4]) @ qml.PauliZ(wires=[3]),
+        ]
+
+        coefs = (np.random.rand(len(obs)) - 0.5) * 2
+        hamiltonian = qml.Hamiltonian(coefs, obs)
+
+        cost = catch_warn_ExpvalCost(
+            qml.templates.StronglyEntanglingLayers,
+            hamiltonian,
+            dev,
+            optimize=True,
+            interface="torch",
+            diff_method="parameter-shift",
+        )
+        cost2 = catch_warn_ExpvalCost(
+            qml.templates.StronglyEntanglingLayers,
+            hamiltonian,
+            dev,
+            optimize=False,
+            interface="torch",
+            diff_method="parameter-shift",
+        )
+
+        np.random.seed(1967)
+        shape = qml.templates.StronglyEntanglingLayers.shape(n_layers=2, n_wires=5)
+        w = np.random.random(shape)
+
+        c1 = cost(w)
+        exec_opt = dev.num_executions
+        dev._num_executions = 0
+
+        c2 = cost2(w)
+        exec_no_opt = dev.num_executions
+
+        assert exec_opt == 1  # Number of groups in the Hamiltonian
+        assert exec_no_opt == 8
+
+        assert np.allclose(c1, c2)
+
+    @pytest.mark.tf
+    def test_optimize_multiple_terms_tf(self):
+        """Test that an ExpvalCost with observable optimization gives the same
+        result as another ExpvalCost without observable optimization even when there
+        are non-unique Hamiltonian terms."""
+
+        dev = qml.device("default.qubit", wires=5)
+        obs = [
+            qml.PauliZ(wires=[2]) @ qml.PauliZ(wires=[4]),  # <---- These two terms
+            qml.PauliZ(wires=[4]) @ qml.PauliZ(wires=[2]),  # <---- are equal
+            qml.PauliZ(wires=[1]),
+            qml.PauliZ(wires=[2]),
+            qml.PauliZ(wires=[1]) @ qml.PauliZ(wires=[2]),
+            qml.PauliZ(wires=[2]) @ qml.PauliZ(wires=[0]),
+            qml.PauliZ(wires=[3]) @ qml.PauliZ(wires=[1]),
+            qml.PauliZ(wires=[4]) @ qml.PauliZ(wires=[3]),
+        ]
+
+        coefs = (np.random.rand(len(obs)) - 0.5) * 2
+        hamiltonian = qml.Hamiltonian(coefs, obs)
+
+        cost = catch_warn_ExpvalCost(
+            qml.templates.StronglyEntanglingLayers,
+            hamiltonian,
+            dev,
+            optimize=True,
+            interface="tf",
+            diff_method="parameter-shift",
+        )
+        cost2 = catch_warn_ExpvalCost(
+            qml.templates.StronglyEntanglingLayers,
+            hamiltonian,
+            dev,
+            optimize=False,
+            interface="tf",
+            diff_method="parameter-shift",
+        )
+
+        np.random.seed(1967)
+        shape = qml.templates.StronglyEntanglingLayers.shape(n_layers=2, n_wires=5)
+        w = np.random.random(shape)
+
+        c1 = cost(w)
+        exec_opt = dev.num_executions
+        dev._num_executions = 0
+
+        c2 = cost2(w)
+        exec_no_opt = dev.num_executions
+
+        assert exec_opt == 1  # Number of groups in the Hamiltonian
+        assert exec_no_opt == 8
+
+        assert np.allclose(c1, c2)
+
+    @pytest.mark.autograd
     def test_optimize_grad(self):
         """Test that the gradient of ExpvalCost is accessible and correct when using observable
         optimization and the autograd interface."""
         dev = qml.device("default.qubit", wires=4)
         hamiltonian = big_hamiltonian
 
-        cost = qml.ExpvalCost(
+        cost = catch_warn_ExpvalCost(
             qml.templates.StronglyEntanglingLayers,
             hamiltonian,
             dev,
             optimize=True,
             diff_method="parameter-shift",
         )
-        cost2 = qml.ExpvalCost(
+        cost2 = catch_warn_ExpvalCost(
             qml.templates.StronglyEntanglingLayers,
             hamiltonian,
             dev,
@@ -495,7 +679,7 @@ class TestVQE:
 
         np.random.seed(1967)
         shape = qml.templates.StronglyEntanglingLayers.shape(n_layers=2, n_wires=4)
-        w = np.random.uniform(low=0, high=2 * np.pi, size=shape)
+        w = pnp.random.uniform(low=0, high=2 * np.pi, size=shape, requires_grad=True)
 
         dc = qml.grad(cost)(w)
         exec_opt = dev.num_executions
@@ -508,6 +692,7 @@ class TestVQE:
         assert np.allclose(dc, big_hamiltonian_grad)
         assert np.allclose(dc2, big_hamiltonian_grad)
 
+    @pytest.mark.autograd
     @pytest.mark.parametrize("opt", [True, False])
     def test_grad_zero_hamiltonian(self, opt):
         """Test that the gradient of ExpvalCost is accessible and correct when using observable
@@ -515,7 +700,7 @@ class TestVQE:
         dev = qml.device("default.qubit", wires=4)
         hamiltonian = qml.Hamiltonian([0], [qml.PauliX(0)])
 
-        cost = qml.ExpvalCost(
+        cost = catch_warn_ExpvalCost(
             qml.templates.StronglyEntanglingLayers,
             hamiltonian,
             dev,
@@ -525,22 +710,23 @@ class TestVQE:
 
         np.random.seed(1967)
         shape = qml.templates.StronglyEntanglingLayers.shape(n_layers=2, n_wires=4)
-        w = np.random.random(shape)
+        w = pnp.random.random(shape, requires_grad=True)
 
-        dc = qml.grad(cost)(w)
+        with pytest.warns(UserWarning, match="Output seems independent of input"):
+            dc = qml.grad(cost)(w)
         assert np.allclose(dc, 0)
 
+    @pytest.mark.torch
     @pytest.mark.slow
-    def test_optimize_grad_torch(self, torch_support):
+    def test_optimize_grad_torch(self):
         """Test that the gradient of ExpvalCost is accessible and correct when using observable
         optimization and the Torch interface."""
-        if not torch_support:
-            pytest.skip("This test requires Torch")
+        import torch
 
         dev = qml.device("default.qubit", wires=4)
         hamiltonian = big_hamiltonian
 
-        cost = qml.ExpvalCost(
+        cost = catch_warn_ExpvalCost(
             qml.templates.StronglyEntanglingLayers,
             hamiltonian,
             dev,
@@ -559,17 +745,17 @@ class TestVQE:
 
         assert np.allclose(dc, big_hamiltonian_grad)
 
+    @pytest.mark.tf
     @pytest.mark.slow
-    def test_optimize_grad_tf(self, tf_support):
+    def test_optimize_grad_tf(self):
         """Test that the gradient of ExpvalCost is accessible and correct when using observable
         optimization and the TensorFlow interface."""
-        if not tf_support:
-            pytest.skip("This test requires TensorFlow")
+        import tensorflow as tf
 
         dev = qml.device("default.qubit", wires=4)
         hamiltonian = big_hamiltonian
 
-        cost = qml.ExpvalCost(
+        cost = catch_warn_ExpvalCost(
             qml.templates.StronglyEntanglingLayers, hamiltonian, dev, optimize=True, interface="tf"
         )
 
@@ -585,11 +771,12 @@ class TestVQE:
 
         assert np.allclose(dc, big_hamiltonian_grad)
 
-    def test_metric_tensor(self):
+    @pytest.mark.parametrize("approx", [None, "block-diag", "diag"])
+    def test_metric_tensor(self, approx):
         """Test that the metric tensor can be calculated."""
 
-        dev = qml.device("default.qubit", wires=2)
-        p = np.array([1.0, 1.0, 1.0])
+        dev = qml.device("default.qubit", wires=3)
+        p = pnp.array([1.0, 1.0, 1.0], requires_grad=True)
 
         def ansatz(params, **kwargs):
             qml.RX(params[0], wires=0)
@@ -598,10 +785,10 @@ class TestVQE:
             qml.PhaseShift(params[2], wires=1)
 
         h = qml.Hamiltonian([1, 1], [qml.PauliZ(0), qml.PauliZ(1)])
-        qnodes = qml.ExpvalCost(ansatz, h, dev)
-        mt = qml.metric_tensor(qnodes)(p)
+        qnodes = catch_warn_ExpvalCost(ansatz, h, dev)
+        mt = qml.metric_tensor(qnodes, approx=approx)(p)
         assert mt.shape == (3, 3)
-        assert isinstance(mt, np.ndarray)
+        assert isinstance(mt, pnp.ndarray)
 
     def test_multiple_devices(self, mocker):
         """Test that passing multiple devices to ExpvalCost works correctly"""
@@ -613,9 +800,10 @@ class TestVQE:
         obs = [qml.PauliZ(0), qml.PauliZ(1)]
         h = qml.Hamiltonian([1, 1], obs)
 
-        qnodes = qml.ExpvalCost(qml.templates.BasicEntanglerLayers, h, dev)
+        qnodes = catch_warn_ExpvalCost(qml.templates.BasicEntanglerLayers, h, dev)
         np.random.seed(1967)
         w = np.random.random(qml.templates.BasicEntanglerLayers.shape(n_layers=3, n_wires=2))
+        w = pnp.array(w, requires_grad=True)
 
         res = qnodes(w)
 
@@ -628,7 +816,7 @@ class TestVQE:
         assert np.allclose(res, exp)
 
         with pytest.warns(UserWarning, match="ExpvalCost was instantiated with multiple devices."):
-            qml.metric_tensor(qnodes)(w)
+            qml.metric_tensor(qnodes, approx="block-diag")(w)
 
     def test_multiple_devices_opt_true(self):
         """Test if a ValueError is raised when multiple devices are passed when optimize=True."""
@@ -637,7 +825,7 @@ class TestVQE:
         h = qml.Hamiltonian([1, 1], [qml.PauliZ(0), qml.PauliZ(1)])
 
         with pytest.raises(ValueError, match="Using multiple devices is not supported when"):
-            qml.ExpvalCost(qml.templates.StronglyEntanglingLayers, h, dev, optimize=True)
+            catch_warn_ExpvalCost(qml.templates.StronglyEntanglingLayers, h, dev, optimize=True)
 
     def test_variance_error(self):
         """Test that an error is raised if attempting to use ExpvalCost to measure
@@ -646,7 +834,9 @@ class TestVQE:
         hamiltonian = big_hamiltonian
 
         with pytest.raises(ValueError, match="sums of expectation values"):
-            qml.ExpvalCost(qml.templates.StronglyEntanglingLayers, hamiltonian, dev, measure="var")
+            catch_warn_ExpvalCost(
+                qml.templates.StronglyEntanglingLayers, hamiltonian, dev, measure="var"
+            )
 
 
 # Test data
@@ -716,6 +906,7 @@ class TestNewVQE:
 
         assert np.allclose(res1, res2, atol=tol)
 
+    @pytest.mark.autograd
     @pytest.mark.parametrize("shots, dim", [([(1000, 2)], 2), ([30, 30], 2), ([2, 3, 4], 3)])
     def test_shot_distribution(self, shots, dim):
         """Tests that distributed shots work with the new VQE design."""
@@ -749,7 +940,7 @@ class TestNewVQE:
             return qml.expval(H1)
 
         res = qml.draw(circuit1)()
-        expected = " 0: ──H──╭┤ ⟨Hamiltonian(1, 1, 1)⟩ \n" + " 2: ─────╰┤ ⟨Hamiltonian(1, 1, 1)⟩ \n"
+        expected = "0: ──H─┤ ╭<𝓗(1.00,1.00,1.00)>\n2: ────┤ ╰<𝓗(1.00,1.00,1.00)>"
         assert res == expected
 
     def test_multiple_expvals(self):
@@ -808,7 +999,9 @@ class TestNewVQE:
         def circuit():
             return qml.var(H)
 
-        with pytest.raises(ValueError, match="Cannot compute analytic variance"):
+        with pytest.raises(
+            qml.operation.EigvalsUndefinedError, match="Cannot compute analytic variance"
+        ):
             circuit()
 
     def test_error_sample_measurement(self):
@@ -825,6 +1018,7 @@ class TestNewVQE:
         with pytest.raises(ValueError, match="Can only return the expectation of a single"):
             circuit()
 
+    @pytest.mark.autograd
     @pytest.mark.parametrize("diff_method", ["parameter-shift", "best"])
     def test_grad_autograd(self, diff_method, tol):
         """Tests the VQE gradient in the autograd interface."""
@@ -840,6 +1034,7 @@ class TestNewVQE:
         dc = qml.grad(circuit)(w)
         assert np.allclose(dc, big_hamiltonian_grad, atol=tol)
 
+    @pytest.mark.autograd
     def test_grad_zero_hamiltonian(self, tol):
         """Tests the VQE gradient for a "zero" Hamiltonian."""
         dev = qml.device("default.qubit", wires=4)
@@ -854,11 +1049,11 @@ class TestNewVQE:
         dc = qml.grad(circuit)(w)
         assert np.allclose(dc, 0, atol=tol)
 
+    @pytest.mark.torch
     @pytest.mark.slow
-    def test_grad_torch(self, torch_support, tol):
+    def test_grad_torch(self, tol):
         """Tests VQE gradients in the torch interface."""
-        if not torch_support:
-            pytest.skip("This test requires Torch")
+        import torch
 
         dev = qml.device("default.qubit", wires=4)
         H = big_hamiltonian
@@ -876,10 +1071,10 @@ class TestNewVQE:
 
         assert np.allclose(dc, big_hamiltonian_grad, atol=tol)
 
-    def test_grad_tf(self, tf_support, tol):
+    @pytest.mark.tf
+    def test_grad_tf(self, tol):
         """Tests VQE gradients in the tf interface."""
-        if not tf_support:
-            pytest.skip("This test requires TensorFlow")
+        import tensorflow as tf
 
         dev = qml.device("default.qubit", wires=4)
         H = big_hamiltonian
@@ -898,10 +1093,11 @@ class TestNewVQE:
 
         assert np.allclose(dc, big_hamiltonian_grad, atol=tol)
 
+    @pytest.mark.jax
     @pytest.mark.slow
     def test_grad_jax(self, tol):
         """Tests VQE gradients in the jax interface."""
-        jax = pytest.importorskip("jax")
+        import jax
         from jax import numpy as jnp
 
         dev = qml.device("default.qubit", wires=4)
@@ -935,6 +1131,7 @@ class TestNewVQE:
         assert res["num_used_wires"] == 2
 
 
+@pytest.mark.autograd
 class TestAutogradInterface:
     """Tests for the Autograd interface (and the NumPy interface for backward compatibility)"""
 
@@ -951,6 +1148,7 @@ class TestAutogradInterface:
         res = [c(params) for c in circuits]
         assert all(isinstance(val, (np.ndarray, float)) for val in res)
 
+    @pytest.mark.autograd
     @pytest.mark.parametrize("interface", ["autograd"])
     def test_gradient(self, tol, interface):
         """Test differentiation works"""
@@ -967,7 +1165,7 @@ class TestAutogradInterface:
         a, b = 0.54, 0.123
         params = np.array([a, b])
 
-        cost = qml.ExpvalCost(ansatz, H, dev, interface=interface)
+        cost = catch_warn_ExpvalCost(ansatz, H, dev, interface=interface)
         dcost = qml.grad(cost, argnum=[0])
         res = dcost(params)
 
@@ -979,7 +1177,7 @@ class TestAutogradInterface:
         assert np.allclose(res, expected, atol=tol, rtol=0)
 
 
-@pytest.mark.usefixtures("skip_if_no_torch_support")
+@pytest.mark.torch
 class TestTorchInterface:
     """Tests for the PyTorch interface"""
 
@@ -987,6 +1185,8 @@ class TestTorchInterface:
     @pytest.mark.parametrize("observables", OBSERVABLES)
     def test_QNodes_have_right_interface(self, ansatz, observables, params, mock_device):
         """Test that QNodes have the torch interface"""
+        import torch
+
         dev = mock_device(wires=3)
         circuits = qml.map(ansatz, observables, device=dev, interface="torch")
         assert all(c.interface == "torch" for c in circuits)
@@ -996,6 +1196,8 @@ class TestTorchInterface:
 
     def test_gradient(self, tol):
         """Test differentiation works"""
+        import torch
+
         dev = qml.device("default.qubit", wires=1)
 
         def ansatz(params, **kwargs):
@@ -1009,7 +1211,7 @@ class TestTorchInterface:
         a, b = 0.54, 0.123
         params = torch.autograd.Variable(torch.tensor([a, b]), requires_grad=True)
 
-        cost = qml.ExpvalCost(ansatz, H, dev, interface="torch")
+        cost = catch_warn_ExpvalCost(ansatz, H, dev, interface="torch")
         loss = cost(params)
         loss.backward()
 
@@ -1023,7 +1225,7 @@ class TestTorchInterface:
         assert np.allclose(res, expected, atol=tol, rtol=0)
 
 
-@pytest.mark.usefixtures("skip_if_no_tf_support")
+@pytest.mark.tf
 class TestTFInterface:
     """Tests for the TF interface"""
 
@@ -1031,6 +1233,8 @@ class TestTFInterface:
     @pytest.mark.parametrize("observables", OBSERVABLES)
     def test_QNodes_have_right_interface(self, ansatz, observables, params, mock_device):
         """Test that QNodes have the tf interface"""
+        import tensorflow as tf
+
         if ansatz == amp_embed_and_strong_ent_layer:
             pytest.skip("TF doesn't work with ragged arrays")
 
@@ -1039,10 +1243,12 @@ class TestTFInterface:
         assert all(c.interface == "tf" for c in circuits)
 
         res = [c(params) for c in circuits]
-        assert all(isinstance(val, (Variable, tf.Tensor)) for val in res)
+        assert all(isinstance(val, (tf.Variable, tf.Tensor)) for val in res)
 
     def test_gradient(self, tol):
         """Test differentiation works"""
+        import tensorflow as tf
+
         dev = qml.device("default.qubit", wires=1)
 
         def ansatz(params, **kwargs):
@@ -1054,8 +1260,8 @@ class TestTFInterface:
 
         H = qml.Hamiltonian(coeffs, observables)
         a, b = 0.54, 0.123
-        params = Variable([a, b], dtype=tf.float64)
-        cost = qml.ExpvalCost(ansatz, H, dev, interface="tf")
+        params = tf.Variable([a, b], dtype=tf.float64)
+        cost = catch_warn_ExpvalCost(ansatz, H, dev, interface="tf")
 
         with tf.GradientTape() as tape:
             loss = cost(params)
@@ -1069,13 +1275,16 @@ class TestTFInterface:
         assert np.allclose(res, expected, atol=tol, rtol=0)
 
 
-@pytest.mark.usefixtures("skip_if_no_tf_support")
-@pytest.mark.usefixtures("skip_if_no_torch_support")
+# Multiple interfaces it will bee tested with math module
+@pytest.mark.all_interfaces
 class TestMultipleInterfaceIntegration:
     """Tests to ensure that interfaces agree and integrate correctly"""
 
     def test_all_interfaces_gradient_agree(self, tol):
         """Test the gradient agrees across all interfaces"""
+        import torch
+        import tensorflow as tf
+
         dev = qml.device("default.qubit", wires=2)
 
         coeffs = [0.2, 0.5]
@@ -1088,10 +1297,10 @@ class TestMultipleInterfaceIntegration:
         params = np.random.uniform(low=0, high=2 * np.pi, size=shape)
 
         # TensorFlow interface
-        w = Variable(params)
+        w = tf.Variable(params)
         ansatz = qml.templates.layers.StronglyEntanglingLayers
 
-        cost = qml.ExpvalCost(ansatz, H, dev, interface="tf")
+        cost = catch_warn_ExpvalCost(ansatz, H, dev, interface="tf")
 
         with tf.GradientTape() as tape:
             loss = cost(w)
@@ -1102,7 +1311,7 @@ class TestMultipleInterfaceIntegration:
         w = torch.autograd.Variable(w, requires_grad=True)
         ansatz = qml.templates.layers.StronglyEntanglingLayers
 
-        cost = qml.ExpvalCost(ansatz, H, dev, interface="torch")
+        cost = catch_warn_ExpvalCost(ansatz, H, dev, interface="torch")
         loss = cost(w)
         loss.backward()
         res_torch = w.grad.numpy()
@@ -1110,7 +1319,7 @@ class TestMultipleInterfaceIntegration:
         # NumPy interface
         w = params
         ansatz = qml.templates.layers.StronglyEntanglingLayers
-        cost = qml.ExpvalCost(ansatz, H, dev, interface="autograd")
+        cost = catch_warn_ExpvalCost(ansatz, H, dev, interface="autograd")
         dcost = qml.grad(cost, argnum=[0])
         res = dcost(w)
 
